// For GNOME Shell version before 45
class Extension {
    constructor(meta) { // meta has type ExtensionMeta
      this.metadata = meta.metadata;
      this.uuid = meta.uuid;
      this.path = meta.path;
    }
    getSettings() {
        return imports.misc.extensionUtils.getSettings();
    }
}

class Mtk { Rectangle }
Mtk.Rectangle = function (params = {}) {
    return new imports.gi.Meta.Rectangle(params);
};
Mtk.Rectangle.$gtype = imports.gi.Meta.Rectangle.$gtype;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// src/utils/shell.ts
var logger = (prefix) => (...content) => console.log("[tilingshell]", `[${prefix}]`, ...content);

// src/utils/ui.ts
const St = imports.gi.St;
const Meta = imports.gi.Meta;
const Clutter = imports.gi.Clutter;
;
const Main = imports.ui.main;
var getMonitors = () => Main.layoutManager.monitors;
var isPointInsideRect = (point, rect) => {
  return point.x >= rect.x && point.x <= rect.x + rect.width && point.y >= rect.y && point.y <= rect.y + rect.height;
};
var positionRelativeTo = (actor, anchestor) => {
  if (!actor)
    return void 0;
  if (actor === anchestor)
    return { x: actor.x, y: actor.y };
  const parent = actor.get_parent();
  if (parent === null)
    return void 0;
  const parentPos = positionRelativeTo(parent, anchestor);
  if (!parentPos)
    return void 0;
  return {
    x: actor.x + parentPos.x,
    y: actor.y + parentPos.y
  };
};
var buildTileGaps = (tilePos, innerGaps, outerGaps, container, scalingFactor = 1) => {
  const isLeft = tilePos.x === container.x;
  const isTop = tilePos.y === container.y;
  const isRight = tilePos.x + tilePos.width === container.x + container.width;
  const isBottom = tilePos.y + tilePos.height === container.y + container.height;
  const margin = new Clutter.Margin();
  margin.top = (isTop ? outerGaps.top : innerGaps.top / 2) * scalingFactor;
  margin.bottom = (isBottom ? outerGaps.bottom : innerGaps.bottom / 2) * scalingFactor;
  margin.left = (isLeft ? outerGaps.left : innerGaps.left / 2) * scalingFactor;
  margin.right = (isRight ? outerGaps.right : innerGaps.right / 2) * scalingFactor;
  return margin;
};
var getScalingFactor = (monitorIndex) => {
  const scalingFactor = St.ThemeContext.get_for_stage(global.get_stage()).get_scale_factor();
  if (scalingFactor === 1)
    return global.display.get_monitor_scale(monitorIndex);
  return scalingFactor;
};
var getScalingFactorOf = (widget) => {
  const [hasReference, scalingReference] = widget.get_theme_node().lookup_length("scaling-reference", true);
  if (!hasReference)
    return [true, 1];
  const [hasValue, monitorScalingFactor] = widget.get_theme_node().lookup_length("monitor-scaling-factor", true);
  if (!hasValue)
    return [true, 1];
  return [scalingReference != 1, monitorScalingFactor / scalingReference];
};
var enableScalingFactorSupport = (widget, monitorScalingFactor) => {
  if (!monitorScalingFactor)
    return;
  widget.set_style(`scaling-reference: 1px; monitor-scaling-factor: ${monitorScalingFactor}px;`);
};
function getWindowsOfMonitor(monitor) {
  let windows = global.workspaceManager.get_active_workspace().list_windows().filter((win) => win.get_window_type() === Meta.WindowType.NORMAL && Main.layoutManager.monitors[win.get_monitor()] === monitor);
  return windows;
}
function buildMarginOf(value) {
  const margin = new Clutter.Margin();
  margin.top = value;
  margin.bottom = value;
  margin.left = value;
  margin.right = value;
  return margin;
}
function buildMargin(params) {
  const margin = new Clutter.Margin();
  if (params.top)
    margin.top = params.top;
  if (params.bottom)
    margin.bottom = params.bottom;
  if (params.left)
    margin.left = params.left;
  if (params.right)
    margin.right = params.right;
  return margin;
}
function buildRectangle(params = {}) {
  return new Mtk.Rectangle({ x: params.x || 0, y: params.y || 0, width: params.width || 0, height: params.height || 0 });
}
function getEventCoords(event) {
  return event.get_coords ? event.get_coords() : [event.x, event.y];
}

// src/utils/gjs.ts
const GObject = imports.gi.GObject;
function registerGObjectClass(target) {
  if (Object.prototype.hasOwnProperty.call(target, "metaInfo")) {
    return GObject.registerClass(target.metaInfo, target);
  } else {
    return GObject.registerClass(target);
  }
}

// src/components/tilepreview/tilePreview.ts
const St2 = imports.gi.St;
const Clutter2 = imports.gi.Clutter;
var WINDOW_ANIMATION_TIME = 100;
var debug = logger("tilePreview");
var TilePreview = class extends St2.Widget {
  _rect;
  _showing;
  _gaps;
  constructor(params) {
    super(params);
    if (params.parent)
      params.parent.add_child(this);
    this._showing = false;
    this._rect = params.rect || buildRectangle({});
    this._gaps = new Clutter2.Margin();
    this.gaps = params.gaps || new Clutter2.Margin();
  }
  set gaps(gaps) {
    const [_, scalingFactor] = getScalingFactorOf(this);
    this._gaps.top = gaps.top * scalingFactor;
    this._gaps.right = gaps.right * scalingFactor;
    this._gaps.bottom = gaps.bottom * scalingFactor;
    this._gaps.left = gaps.left * scalingFactor;
  }
  get gaps() {
    return this._gaps;
  }
  _init() {
    super._init();
    this.set_style_class_name("tile-preview custom-tile-preview");
    this.hide();
  }
  get innerX() {
    return this._rect.x + this._gaps.left;
  }
  get innerY() {
    return this._rect.y + this._gaps.top;
  }
  get innerWidth() {
    return this._rect.width - this._gaps.right - this._gaps.left;
  }
  get innerHeight() {
    return this._rect.height - this._gaps.top - this._gaps.bottom;
  }
  get rect() {
    return this._rect;
  }
  get showing() {
    return this._showing;
  }
  open(ease = false, position) {
    if (position)
      this._rect = position;
    const fadeInMove = this._showing;
    this._showing = true;
    this.show();
    if (fadeInMove) {
      this.ease({
        x: this.innerX,
        y: this.innerY,
        width: this.innerWidth,
        height: this.innerHeight,
        opacity: 255,
        duration: ease ? WINDOW_ANIMATION_TIME : 0,
        mode: Clutter2.AnimationMode.EASE_OUT_QUAD
      });
    } else {
      this.set_position(this.innerX, this.innerY);
      this.set_size(this.innerWidth, this.innerHeight);
      this.ease({
        opacity: 255,
        duration: ease ? WINDOW_ANIMATION_TIME : 0,
        mode: Clutter2.AnimationMode.EASE_OUT_QUAD
      });
    }
  }
  openBelow(window, ease = false, position) {
    if (this.get_parent() === global.windowGroup) {
      let windowActor = window.get_compositor_private();
      if (!windowActor)
        return;
      global.windowGroup.set_child_below_sibling(this, windowActor);
    }
    this.open(ease, position);
  }
  openAbove(window, ease = false, position) {
    if (this.get_parent() === global.windowGroup) {
      let windowActor = window.get_compositor_private();
      if (!windowActor)
        return;
      global.windowGroup.set_child_above_sibling(this, windowActor);
    }
    this.open(ease, position);
  }
  close() {
    if (!this._showing)
      return;
    this._showing = false;
    this.ease({
      opacity: 0,
      duration: WINDOW_ANIMATION_TIME,
      mode: Clutter2.AnimationMode.EASE_OUT_QUAD,
      onComplete: () => this.hide()
    });
  }
};
TilePreview = __decorateClass([
  registerGObjectClass
], TilePreview);

// src/components/layout/Layout.ts
var Layout = class {
  id;
  tiles;
  constructor(tiles, id) {
    this.tiles = tiles;
    this.id = id;
  }
};

// src/components/layout/Tile.ts
const GObject2 = imports.gi.GObject;
var Tile2 = class {
  //@ts-ignore todo
  static $gtype = GObject2.TYPE_JSOBJECT;
  x;
  y;
  width;
  height;
  groups;
  constructor({ x, y, width, height, groups }) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.groups = groups;
  }
};

// src/components/layout/TileUtils.ts
var TileUtils = class {
  static apply_props(tile, container) {
    return buildRectangle({
      x: Math.round(container.width * tile.x + container.x),
      y: Math.round(container.height * tile.y + container.y),
      width: Math.round(container.width * tile.width),
      height: Math.round(container.height * tile.height)
    });
  }
  static build_tile(rect, container) {
    return new Tile2({
      x: (rect.x - container.x) / container.width,
      y: (rect.y - container.y) / container.height,
      width: rect.width / container.width,
      height: rect.height / container.height,
      groups: []
    });
  }
};

// src/components/layout/LayoutWidget.ts
const St3 = imports.gi.St;
const Clutter3 = imports.gi.Clutter;
var debug2 = logger(`LayoutWidget`);
var LayoutWidget = class extends St3.Widget {
  _previews;
  _containerRect;
  _layout;
  _innerGaps;
  _outerGaps;
  constructor(params) {
    super({ styleClass: params.styleClass || "" });
    params.parent.add_child(this);
    if (params.scalingFactor)
      this.scalingFactor = params.scalingFactor;
    this._previews = [];
    this._containerRect = params.containerRect || buildRectangle();
    this._layout = params.layout || new Layout([], "");
    this._innerGaps = params.innerGaps || new Clutter3.Margin();
    this._outerGaps = params.outerGaps || new Clutter3.Margin();
  }
  set scalingFactor(value) {
    enableScalingFactorSupport(this, value);
  }
  get innerGaps() {
    return this._innerGaps.copy();
  }
  get outerGaps() {
    return this._outerGaps.copy();
  }
  draw_layout() {
    this._previews = this._layout.tiles.map((tile) => {
      const tileRect = TileUtils.apply_props(tile, this._containerRect);
      const tileMargin = buildTileGaps(tileRect, this._innerGaps, this._outerGaps, this._containerRect);
      return this.buildTile(this, tileRect, tileMargin, tile);
    });
  }
  buildTile(parent, rect, margin, tile) {
    throw "This class shouldn't be instantiated but it should be extended instead";
  }
  relayout(params) {
    var trigger_relayout = this._previews.length === 0;
    if (params?.innerGaps) {
      this._innerGaps = params.innerGaps.copy();
      trigger_relayout = true;
    }
    if (params?.outerGaps && this._outerGaps !== params.outerGaps) {
      this._outerGaps = params.outerGaps.copy();
      trigger_relayout = true;
    }
    if (params?.layout && this._layout !== params.layout) {
      this._layout = params.layout;
      trigger_relayout = true;
    }
    if (params?.containerRect && this._containerRect !== params.containerRect) {
      this._containerRect = params.containerRect.copy();
      trigger_relayout = true;
    }
    if (!trigger_relayout) {
      debug2("relayout not needed");
      return false;
    }
    this._previews?.forEach((preview) => {
      if (preview.get_parent() === this) {
        this.remove_child(preview);
      }
      preview.destroy();
    });
    this._previews = [];
    if (this._containerRect.width === 0 || this._containerRect.height === 0)
      return true;
    this.draw_layout();
    this._previews.forEach((lay) => lay.open());
    return true;
  }
};
LayoutWidget = __decorateClass([
  registerGObjectClass
], LayoutWidget);

// src/components/tilingsystem/tilingLayout.ts
const Clutter4 = imports.gi.Clutter;
var debug3 = logger("tilingLayout");
var DynamicTilePreview = class extends TilePreview {
  _originalRect;
  _canRestore;
  constructor(params, canRestore) {
    super(params);
    this._canRestore = canRestore || false;
    this._originalRect = this.rect.copy();
  }
  get originalRect() {
    return this._originalRect;
  }
  get canRestore() {
    return this._canRestore;
  }
  restore(ease = false) {
    if (!this._canRestore)
      return false;
    this._rect = this._originalRect.copy();
    if (this.showing)
      this.open(ease);
    return true;
  }
};
DynamicTilePreview = __decorateClass([
  registerGObjectClass
], DynamicTilePreview);
var TilingLayout = class extends LayoutWidget {
  _showing;
  constructor(layout, innerGaps, outerGaps, workarea, scalingFactor) {
    super({
      containerRect: workarea,
      parent: global.windowGroup,
      layout,
      innerGaps,
      outerGaps,
      scalingFactor
    });
    this._showing = false;
    super.relayout();
  }
  _init() {
    super._init();
    this.hide();
  }
  buildTile(parent, rect, gaps, tile) {
    return new DynamicTilePreview({ parent, rect, gaps }, true);
  }
  get showing() {
    return this._showing;
  }
  openBelow(window) {
    if (this._showing)
      return;
    let windowActor = window.get_compositor_private();
    if (!windowActor)
      return;
    global.windowGroup.set_child_below_sibling(this, windowActor);
    this.open();
  }
  openAbove(window) {
    if (this._showing)
      return;
    let windowActor = window.get_compositor_private();
    if (!windowActor)
      return;
    global.windowGroup.set_child_above_sibling(this, windowActor);
    this.open();
  }
  open() {
    if (this._showing)
      return;
    this.show();
    this._showing = true;
    this.ease({
      x: this.x,
      y: this.y,
      opacity: 255,
      duration: WINDOW_ANIMATION_TIME,
      mode: Clutter4.AnimationMode.EASE_OUT_QUAD
    });
  }
  close() {
    if (!this._showing)
      return;
    this._showing = false;
    this.ease({
      opacity: 0,
      duration: WINDOW_ANIMATION_TIME,
      mode: Clutter4.AnimationMode.EASE_OUT_QUAD,
      onComplete: () => {
        this.unhoverAllTiles();
        this.hide();
      }
    });
  }
  _isHovered(currPointerPos, preview) {
    return currPointerPos.x >= preview.x && currPointerPos.x <= preview.x + preview.width && currPointerPos.y >= preview.y && currPointerPos.y <= preview.y + preview.height;
  }
  getTileBelow(currPointerPos, reset) {
    let found = this._previews.find((preview) => this._isHovered(currPointerPos, preview.rect));
    if (!found || !found.canRestore && reset) {
      found = this._previews.find((preview) => preview.canRestore && this._isHovered(currPointerPos, preview.originalRect));
    }
    if (!found)
      return void 0;
    if (reset && found.originalRect)
      return found.originalRect;
    return found.rect;
  }
  unhoverAllTiles() {
    const newPreviewsArray = [];
    this._previews.forEach((preview) => {
      if (preview.restore(true)) {
        newPreviewsArray.push(preview);
        preview.open();
      } else {
        this.remove_child(preview);
        preview.destroy();
      }
    });
    this._previews = newPreviewsArray;
  }
  hoverTilesInRect(rect, reset) {
    const newPreviewsArray = [];
    this._previews.forEach((preview) => {
      const [hasIntersection, rectangles] = this._subtractRectangles(preview.rect, rect);
      if (hasIntersection) {
        if (rectangles.length > 0) {
          let maxIndex = 0;
          for (let i = 0; i < rectangles.length; i++) {
            if (rectangles[i].area() > rectangles[maxIndex].area())
              maxIndex = i;
          }
          for (let i = 0; i < rectangles.length; i++) {
            if (i === maxIndex)
              continue;
            const rect2 = rectangles[i];
            const gaps = buildTileGaps(rect2, this._innerGaps, this._outerGaps, this._containerRect);
            const innerPreview = new DynamicTilePreview({ parent: this, rect: rect2, gaps }, false);
            innerPreview.open();
            this.set_child_above_sibling(innerPreview, preview);
            newPreviewsArray.push(innerPreview);
          }
          preview.open(false, rectangles[maxIndex].union(preview.rect.intersect(rect)[1]));
          preview.open(true, rectangles[maxIndex]);
          newPreviewsArray.push(preview);
        } else {
          preview.close();
          newPreviewsArray.push(preview);
        }
      } else {
        if (reset) {
          if (preview.restore(true)) {
            preview.open(true);
            newPreviewsArray.push(preview);
          } else {
            this.remove_child(preview);
            preview.destroy();
          }
        } else {
          preview.open(true);
          newPreviewsArray.push(preview);
        }
      }
    });
    this._previews = newPreviewsArray;
  }
  /* 
          Given the source rectangle (made by A, B, C, D and Hole), subtract the hole and obtain A, B, C and D.
          Edge cases:
              - The hole may not be inside the source rect (i.e there is no interstaction). 
              It returns false and an array with the source rectangle only
              - The hole intersects the source rectangle, it returns true and an array with A, B, C and D rectangles.
              Some of A, B, C and D may not be returned if they don't exist
              - The hole is equal to the source rectangle, it returns true and an empty array since A, B, C and D
              rectangles do not exist
  
          Example:
          -------------------------
          |          A            |
          |-----------------------|
          |  B  |   hole    |  C  |
          |-----------------------|
          |          D            |
          -------------------------
      */
  _subtractRectangles(sourceRect, holeRect) {
    const [hasIntersection, intersection] = sourceRect.intersect(holeRect);
    if (!hasIntersection) {
      return [false, [sourceRect]];
    }
    if (intersection.area() >= sourceRect.area() * 0.98) {
      return [true, []];
    }
    const results = [];
    const heightA = intersection.y - sourceRect.y;
    if (heightA > 0) {
      results.push(buildRectangle({
        x: sourceRect.x,
        y: sourceRect.y,
        width: sourceRect.width,
        height: heightA
      }));
    }
    const widthB = intersection.x - sourceRect.x;
    if (widthB > 0 && intersection.height > 0) {
      results.push(buildRectangle({
        x: sourceRect.x,
        y: intersection.y,
        width: widthB,
        height: intersection.height
      }));
    }
    const widthC = sourceRect.x + sourceRect.width - intersection.x - intersection.width;
    if (widthC > 0 && intersection.height > 0) {
      results.push(buildRectangle({
        x: intersection.x + intersection.width,
        y: intersection.y,
        width: widthC,
        height: intersection.height
      }));
    }
    const heightD = sourceRect.y + sourceRect.height - intersection.y - intersection.height;
    if (heightD > 0) {
      results.push(buildRectangle({
        x: sourceRect.x,
        y: intersection.y + intersection.height,
        width: sourceRect.width,
        height: heightD
      }));
    }
    return [true, results];
  }
};
TilingLayout = __decorateClass([
  registerGObjectClass
], TilingLayout);

// src/components/snapassist/snapAssistTile.ts
const St4 = imports.gi.St;
var debug4 = logger("SnapAssistTile");
var SnapAssistTile = class extends TilePreview {
  _tile;
  _styleChangedSignalID;
  constructor(params) {
    super({ parent: params.parent, rect: params.rect, gaps: params.gaps });
    this._tile = params.tile;
    const isLeft = this._tile.x <= 1e-3;
    const isTop = this._tile.y <= 1e-3;
    const isRight = this._tile.x + this._tile.width >= 0.99;
    const isBottom = this._tile.y + this._tile.height >= 0.99;
    const [alreadyScaled, scalingFactor] = getScalingFactorOf(this);
    const radiusValue = (alreadyScaled ? 1 : scalingFactor) * (this.get_theme_node().get_length("border-radius-value") / (alreadyScaled ? scalingFactor : 1));
    const radius = [2, 2, 2, 2];
    if (isTop && isLeft)
      radius[St4.Corner.TOPLEFT] = radiusValue;
    if (isTop && isRight)
      radius[St4.Corner.TOPRIGHT] = radiusValue;
    if (isBottom && isRight)
      radius[St4.Corner.BOTTOMRIGHT] = radiusValue;
    if (isBottom && isLeft)
      radius[St4.Corner.BOTTOMLEFT] = radiusValue;
    this.set_style(
      `
            border-radius: ${radius[St4.Corner.TOPLEFT]}px ${radius[St4.Corner.TOPRIGHT]}px ${radius[St4.Corner.BOTTOMRIGHT]}px ${radius[St4.Corner.BOTTOMLEFT]}px;`
    );
    this._applyStyle();
    this._styleChangedSignalID = St4.ThemeContext.get_for_stage(global.get_stage()).connect(
      "changed",
      () => {
        this._applyStyle();
      }
    );
    this.connect("destroy", () => this.onDestroy());
  }
  _init() {
    super._init();
    this.set_style_class_name("snap-assist-tile button");
  }
  get tile() {
    return this._tile;
  }
  _applyStyle() {
    const [hasColor, { red, green, blue, alpha }] = this.get_theme_node().lookup_color("color", true);
    if (!hasColor)
      return;
    if (red * 0.299 + green * 0.587 + blue * 0.114 > 186) {
      this.remove_style_class_name("dark");
    } else {
      this.add_style_class_name("dark");
    }
  }
  onDestroy() {
    if (this._styleChangedSignalID) {
      St4.ThemeContext.get_for_stage(global.get_stage()).disconnect(this._styleChangedSignalID);
      this._styleChangedSignalID = void 0;
    }
  }
};
SnapAssistTile = __decorateClass([
  registerGObjectClass
], SnapAssistTile);

// src/components/snapassist/snapAssistLayout.ts
const Clutter6 = imports.gi.Clutter;
var debug5 = logger("snapAssistLayout");
var SnapAssistLayout = class extends LayoutWidget {
  // 16:9 ratio. -> (16*this._snapAssistHeight) / 9 and then rounded to int
  constructor(parent, layout, gaps) {
    super({
      parent,
      layout,
      innerGaps: gaps.copy(),
      outerGaps: new Clutter6.Margin(),
      containerRect: buildRectangle(),
      styleClass: "snap-assist-layout"
    });
    const [_, scalingFactor] = getScalingFactorOf(this);
    const width = SnapAssistLayout._snapAssistWidth * scalingFactor;
    const height = SnapAssistLayout._snapAssistHeight * scalingFactor;
    super.relayout({
      containerRect: buildRectangle({ x: 0, y: 0, width, height })
    });
  }
  buildTile(parent, rect, gaps, tile) {
    return new SnapAssistTile({ parent, rect, gaps, tile });
  }
  getTileBelow(cursorPos) {
    const [x, y] = this.get_transformed_position();
    for (let i = 0; i < this._previews.length; i++) {
      let preview = this._previews[i];
      const pos = { x: x + preview.rect.x, y: y + preview.rect.y };
      const isHovering = cursorPos.x >= pos.x && cursorPos.x <= pos.x + preview.rect.width && cursorPos.y >= pos.y && cursorPos.y <= pos.y + preview.rect.height;
      if (isHovering)
        return preview;
    }
  }
};
__publicField(SnapAssistLayout, "_snapAssistHeight", 68);
__publicField(SnapAssistLayout, "_snapAssistWidth", 120);
SnapAssistLayout = __decorateClass([
  registerGObjectClass
], SnapAssistLayout);

// src/settings.ts
const Gio = imports.gi.Gio;
var Settings = class _Settings {
  static _settings;
  static _is_initialized = false;
  static SETTING_LAST_VERSION_INSTALLED = "last-version-installed";
  static SETTING_TILING_SYSTEM = "enable-tiling-system";
  static SETTING_SNAP_ASSIST = "enable-snap-assist";
  static SETTING_SHOW_INDICATOR = "show-indicator";
  static SETTING_INNER_GAPS = "inner-gaps";
  static SETTING_OUTER_GAPS = "outer-gaps";
  static SETTING_SPAN_MULTIPLE_TILES = "enable-span-multiple-tiles";
  static SETTING_LAYOUTS_JSON = "layouts-json";
  static SETTING_SELECTED_LAYOUTS = "selected-layouts";
  static SETTING_RESTORE_WINDOW_ORIGINAL_SIZE = "restore-window-original-size";
  static SETTING_RESIZE_COMPLEMENTING_WINDOWS = "resize-complementing-windows";
  static initialize(settings) {
    if (this._is_initialized)
      return;
    this._is_initialized = true;
    this._settings = settings;
  }
  static destroy() {
    if (this._is_initialized) {
      this._is_initialized = false;
      this._settings = null;
    }
  }
  static bind(key, object, property) {
    this._settings?.bind(key, object, property, Gio.SettingsBindFlags.DEFAULT);
  }
  static get_last_version_installed() {
    return this._settings?.get_uint(this.SETTING_LAST_VERSION_INSTALLED) || -1;
  }
  static get_tiling_system_enabled() {
    return this._settings?.get_boolean(this.SETTING_TILING_SYSTEM) || false;
  }
  static get_snap_assist_enabled() {
    return this._settings?.get_boolean(this.SETTING_SNAP_ASSIST) || false;
  }
  static get_show_indicator() {
    return this._settings?.get_boolean(this.SETTING_SHOW_INDICATOR) || true;
  }
  static get_inner_gaps(scaleFactor = 1) {
    const value = (this._settings?.get_uint(this.SETTING_INNER_GAPS) || 0) * scaleFactor;
    return {
      top: value,
      bottom: value,
      left: value,
      right: value
    };
  }
  static get_outer_gaps(scaleFactor = 1) {
    const value = (this._settings?.get_uint(this.SETTING_OUTER_GAPS) || 0) * scaleFactor;
    return {
      top: value,
      bottom: value,
      left: value,
      right: value
    };
  }
  static get_span_multiple_tiles() {
    return this._settings?.get_boolean(this.SETTING_SPAN_MULTIPLE_TILES) || false;
  }
  static get_layouts_json() {
    try {
      const layouts = JSON.parse(this._settings?.get_string(this.SETTING_LAYOUTS_JSON) || "[]");
      if (layouts.length === 0)
        throw "At least one layout is required";
      return layouts.filter((layout) => layout.tiles.length > 0);
    } catch (ex) {
      this.reset_layouts_json();
      return JSON.parse(this._settings?.get_string(this.SETTING_LAYOUTS_JSON) || "[]");
    }
  }
  static get_selected_layouts() {
    return this._settings?.get_strv(_Settings.SETTING_SELECTED_LAYOUTS) || [];
  }
  static get_restore_window_original_size() {
    return this._settings?.get_boolean(_Settings.SETTING_RESTORE_WINDOW_ORIGINAL_SIZE) || false;
  }
  static get_resize_complementing_windows() {
    return this._settings?.get_boolean(_Settings.SETTING_RESIZE_COMPLEMENTING_WINDOWS) || false;
  }
  static set_last_version_installed(version) {
    this._settings?.set_uint(this.SETTING_LAST_VERSION_INSTALLED, version);
  }
  static reset_layouts_json() {
    this.save_layouts_json([
      new Layout([
        new Tile2({ x: 0, y: 0, height: 0.5, width: 0.22, groups: [1, 2] }),
        // top-left
        new Tile2({ x: 0, y: 0.5, height: 0.5, width: 0.22, groups: [1, 2] }),
        // bottom-left
        new Tile2({ x: 0.22, y: 0, height: 1, width: 0.56, groups: [2, 3] }),
        // center
        new Tile2({ x: 0.78, y: 0, height: 0.5, width: 0.22, groups: [3, 4] }),
        // top-right
        new Tile2({ x: 0.78, y: 0.5, height: 0.5, width: 0.22, groups: [3, 4] })
        // bottom-right
      ], `Layout 1`),
      new Layout([
        new Tile2({ x: 0, y: 0, height: 1, width: 0.22, groups: [1] }),
        new Tile2({ x: 0.22, y: 0, height: 1, width: 0.56, groups: [1, 2] }),
        new Tile2({ x: 0.78, y: 0, height: 1, width: 0.22, groups: [2] })
      ], `Layout 2`),
      new Layout([
        new Tile2({ x: 0, y: 0, height: 1, width: 0.33, groups: [1] }),
        new Tile2({ x: 0.33, y: 0, height: 1, width: 0.67, groups: [1] })
      ], `Layout 3`),
      new Layout([
        new Tile2({ x: 0, y: 0, height: 1, width: 0.67, groups: [1] }),
        new Tile2({ x: 0.67, y: 0, height: 1, width: 0.33, groups: [1] })
      ], `Layout 4`)
    ]);
  }
  static save_layouts_json(layouts) {
    this._settings?.set_string(this.SETTING_LAYOUTS_JSON, JSON.stringify(layouts));
  }
  static save_selected_layouts_json(ids) {
    this._settings?.set_strv(_Settings.SETTING_SELECTED_LAYOUTS, ids);
  }
  static connect(key, func) {
    return this._settings?.connect(`changed::${key}`, func) || -1;
  }
  static disconnect(id) {
    this._settings?.disconnect(id);
  }
};

// src/signalHandling.ts
var SignalHandling = class {
  _signalsIds;
  constructor() {
    this._signalsIds = {};
  }
  connect(obj, key, fun) {
    const signalId = obj.connect(key, fun);
    this._signalsIds[key] = { id: signalId, obj };
  }
  disconnect(obj) {
    if (!obj) {
      Object.keys(this._signalsIds).forEach((key) => {
        this._signalsIds[key].obj.disconnect(this._signalsIds[key].id);
        delete this._signalsIds[key];
      });
    } else {
      const keyFound = Object.keys(this._signalsIds).find((key) => this._signalsIds[key].obj === obj);
      if (keyFound) {
        obj.disconnect(this._signalsIds[keyFound].id);
        delete this._signalsIds[keyFound];
      }
    }
  }
};

// src/globalState.ts
const GObject4 = imports.gi.GObject;
const Main2 = imports.ui.main;
var GlobalState = class extends GObject4.Object {
  _signals;
  _layouts;
  static get() {
    if (!this._instance)
      this._instance = new GlobalState();
    return this._instance;
  }
  static destroy() {
    if (this._instance) {
      this._instance._signals.disconnect();
      this._instance._layouts = [];
      this._instance = null;
    }
  }
  constructor() {
    super();
    this._signals = new SignalHandling();
    this._layouts = Settings.get_layouts_json();
    this._signals.connect(Settings, Settings.SETTING_LAYOUTS_JSON, () => {
      this._layouts = Settings.get_layouts_json();
      this.emit(GlobalState.SIGNAL_LAYOUTS_CHANGED);
    });
  }
  get layouts() {
    return this._layouts;
  }
  addLayout(newLay) {
    this._layouts.push(newLay);
    this.layouts = this._layouts;
  }
  deleteLayout(layoutToDelete) {
    const layFoundIndex = this._layouts.findIndex((lay) => lay.id === layoutToDelete.id);
    if (layFoundIndex == -1)
      return;
    this._layouts.splice(layFoundIndex, 1);
    this.layouts = this._layouts;
    const selectedLayouts = Settings.get_selected_layouts();
    if (layoutToDelete.id === selectedLayouts[Main2.layoutManager.primaryIndex]) {
      selectedLayouts[Main2.layoutManager.primaryIndex] = this._layouts[0].id;
      Settings.save_selected_layouts_json(selectedLayouts);
    }
  }
  editLayout(newLay) {
    const layFoundIndex = this._layouts.findIndex((lay) => lay.id === newLay.id);
    if (layFoundIndex == -1)
      return;
    this._layouts[layFoundIndex] = newLay;
    this.layouts = this._layouts;
  }
  set layouts(layouts) {
    this._layouts = layouts;
    Settings.save_layouts_json(layouts);
    this.emit(GlobalState.SIGNAL_LAYOUTS_CHANGED);
  }
  getSelectedLayoutOfMonitor(monitorIndex) {
    const selectedLayouts = Settings.get_selected_layouts();
    if (monitorIndex < 0 || monitorIndex >= selectedLayouts.length)
      monitorIndex = 0;
    return this._layouts.find((lay) => lay.id === selectedLayouts[monitorIndex]) || this._layouts[0];
  }
};
__publicField(GlobalState, "metaInfo", {
  Signals: {
    "layouts-changed": {
      param_types: []
    }
  },
  GTypeName: "GlobalState"
});
__publicField(GlobalState, "SIGNAL_LAYOUTS_CHANGED", "layouts-changed");
__publicField(GlobalState, "_instance");
GlobalState = __decorateClass([
  registerGObjectClass
], GlobalState);

// src/components/snapassist/snapAssist.ts
const Clutter7 = imports.gi.Clutter;
const St5 = imports.gi.St;
var SNAP_ASSIST_SIGNAL = "snap-assist";
var SNAP_ASSIST_ANIMATION_TIME = 180;
var debug6 = logger("snapAssist");
var SnapAssistContent = class extends St5.BoxLayout {
  // distance from top when the snap assistant is enlarged
  _enlargedVerticalDistance = 24;
  // cursor's max distance from the snap assistant to enlarge it 
  _activationAreaOffset = 4;
  _gaps = 4;
  _container;
  _showing;
  _signals;
  _snapAssistLayouts;
  _isEnlarged = false;
  _hoveredTile;
  _bottomPadding;
  constructor(container) {
    super({
      name: "snap_assist_content",
      xAlign: Clutter7.ActorAlign.CENTER,
      yAlign: Clutter7.ActorAlign.CENTER,
      vertical: false,
      reactive: true,
      styleClass: "popup-menu-content snap-assistant"
    });
    this._container = container;
    this._container.add_child(this);
    this._signals = new SignalHandling();
    this._snapAssistLayouts = [];
    this._isEnlarged = false;
    this._showing = true;
    const [alreadyScaled, finalScalingFactor] = getScalingFactorOf(this);
    this._bottomPadding = (alreadyScaled ? 1 : finalScalingFactor) * (this.get_theme_node().get_padding(St5.Side.BOTTOM) / (alreadyScaled ? finalScalingFactor : 1));
    this.set_style(`
            padding: ${this._bottomPadding}px !important;
        `);
    this._setLayouts(GlobalState.get().layouts);
    this._signals.connect(GlobalState.get(), GlobalState.SIGNAL_LAYOUTS_CHANGED, () => {
      this._setLayouts(GlobalState.get().layouts);
    });
    this.connect("destroy", () => this._signals.disconnect());
    this.close(false);
  }
  close(ease = false) {
    if (!this._showing)
      return;
    this._showing = false;
    this._isEnlarged = false;
    this.set_x(this._container.width / 2 - this.width / 2);
    this.ease({
      y: this._desiredY,
      opacity: 0,
      duration: ease ? SNAP_ASSIST_ANIMATION_TIME : 0,
      mode: Clutter7.AnimationMode.EASE_OUT_QUAD,
      onComplete: () => {
        this.hide();
      }
    });
  }
  get _desiredY() {
    return this._isEnlarged ? this._enlargedVerticalDistance : -this.height + this._bottomPadding;
  }
  open(ease = false) {
    if (!this._showing)
      this.get_parent()?.set_child_above_sibling(this, null);
    this.set_x(this._container.width / 2 - this.width / 2);
    this.show();
    this._showing = true;
    this.ease({
      y: this._desiredY,
      opacity: 255,
      duration: ease ? SNAP_ASSIST_ANIMATION_TIME : 0,
      mode: Clutter7.AnimationMode.EASE_OUT_QUAD
    });
  }
  _setLayouts(layouts) {
    this._snapAssistLayouts.forEach((lay) => lay.destroy());
    this.remove_all_children();
    const [_, scalingFactor] = getScalingFactorOf(this);
    const inner_gaps = Settings.get_inner_gaps(scalingFactor);
    const layoutGaps = buildMargin({
      top: inner_gaps.top === 0 ? 0 : this._gaps,
      bottom: inner_gaps.bottom === 0 ? 0 : this._gaps,
      left: inner_gaps.left === 0 ? 0 : this._gaps,
      right: inner_gaps.right === 0 ? 0 : this._gaps
    });
    this._snapAssistLayouts = layouts.map((lay, ind) => {
      const saLay = new SnapAssistLayout(this, lay, layoutGaps);
      if (ind < layouts.length - 1) {
        this.add_child(new St5.Widget({ width: this._bottomPadding, height: 1 }));
      }
      return saLay;
    });
    this.ensure_style();
    this.set_x(this._container.width / 2 - this.width / 2);
  }
  onMovingWindow(window, ease = false, currPointerPos) {
    const wasEnlarged = this._isEnlarged;
    this.handleOpening(window, ease, currPointerPos);
    if (!this._showing || !this._isEnlarged) {
      if (this._hoveredTile) {
        this._hoveredTile.set_hover(false);
      }
      this._hoveredTile = void 0;
      if (wasEnlarged)
        this._container.emit(SNAP_ASSIST_SIGNAL, new Tile2({ x: 0, y: 0, width: 0, height: 0, groups: [] }));
      return;
    }
    const changed = this.handleTileHovering(currPointerPos);
    if (changed) {
      const tile = this._hoveredTile?.tile || new Tile2({ x: 0, y: 0, width: 0, height: 0, groups: [] });
      this._container.emit(SNAP_ASSIST_SIGNAL, tile);
    }
  }
  handleOpening(window, ease = false, currPointerPos) {
    if (!this._showing) {
      if (this.get_parent() === global.windowGroup) {
        let windowActor = window.get_compositor_private();
        if (!windowActor)
          return;
        global.windowGroup.set_child_above_sibling(this, windowActor);
      }
    }
    const size = this._isEnlarged ? this.height : this.height / 2;
    const isNear = this.isBetween(this._container.x + this.x - this._activationAreaOffset, currPointerPos.x, this._container.x + this.x + this.width + this._activationAreaOffset) && this.isBetween(this._container.y - this._activationAreaOffset, currPointerPos.y, this._container.y + this._enlargedVerticalDistance + size + this._activationAreaOffset);
    if (this._showing && this._isEnlarged === isNear)
      return;
    this._isEnlarged = isNear;
    this.open(ease);
  }
  handleTileHovering(currPointerPos) {
    if (!this._isEnlarged) {
      const changed = this._hoveredTile !== void 0;
      if (this._hoveredTile) {
        this._hoveredTile.set_hover(false);
      }
      this._hoveredTile = void 0;
      return changed;
    }
    var newTileHovered = void 0;
    for (let index = 0; index < this._snapAssistLayouts.length; index++) {
      const snapAssistLay = this._snapAssistLayouts[index];
      newTileHovered = snapAssistLay.getTileBelow(currPointerPos);
      if (newTileHovered) {
        break;
      }
    }
    const tileChanged = newTileHovered !== this._hoveredTile;
    if (tileChanged) {
      this._hoveredTile?.set_hover(false);
      this._hoveredTile = newTileHovered;
    }
    if (this._hoveredTile)
      this._hoveredTile.set_hover(true);
    return tileChanged;
  }
  isBetween(min, num, max) {
    return min <= num && num <= max;
  }
};
SnapAssistContent = __decorateClass([
  registerGObjectClass
], SnapAssistContent);
var SnapAssist = class extends St5.Widget {
  _content;
  constructor(parent, workArea, scalingFactor) {
    super();
    parent.add_child(this);
    this.workArea = workArea;
    this.set_clip(0, 0, workArea.width, workArea.height);
    if (scalingFactor)
      enableScalingFactorSupport(this, scalingFactor);
    this._content = new SnapAssistContent(this);
  }
  set workArea(newWorkArea) {
    this.set_position(newWorkArea.x, newWorkArea.y);
    this.set_width(newWorkArea.width);
    this.set_clip(0, 0, newWorkArea.width, newWorkArea.height);
  }
  onMovingWindow(window, ease = false, currPointerPos) {
    this._content.onMovingWindow(window, ease, currPointerPos);
  }
  close(ease = false) {
    this._content.close(ease);
  }
};
__publicField(SnapAssist, "metaInfo", {
  Signals: {
    "snap-assist": {
      param_types: [Tile2.$gtype]
    }
  },
  GTypeName: "SnapAssist"
});
SnapAssist = __decorateClass([
  registerGObjectClass
], SnapAssist);

// src/components/tilepreview/selectionTilePreview.ts
const St6 = imports.gi.St;
var debug7 = logger("SelectionTilePreview");
var SelectionTilePreview = class extends TilePreview {
  constructor(params) {
    super({ parent: params.parent, name: "SelectionTilePreview" });
    this._recolor();
    const styleChangedSignalID = St6.ThemeContext.get_for_stage(global.get_stage()).connect("changed", () => {
      this.set_style(null);
      this._recolor();
    });
    this.connect("destroy", () => St6.ThemeContext.get_for_stage(global.get_stage()).disconnect(styleChangedSignalID));
  }
  _init() {
    super._init();
    this.add_style_class_name("selection-tile-preview");
  }
  _recolor() {
    const backgroundColor = this.get_theme_node().get_background_color().copy();
    const newAlpha = Math.max(Math.min(backgroundColor.alpha + 35, 255), 160);
    this.set_style(`
      background-color: rgba(${backgroundColor.red}, ${backgroundColor.green}, ${backgroundColor.blue}, ${newAlpha / 255}) !important;
    `);
  }
  close() {
    if (!this._showing)
      return;
    this._rect.width = this.gaps.left + this.gaps.right;
    this._rect.height = this.gaps.top + this.gaps.bottom;
    super.close();
  }
};
SelectionTilePreview = __decorateClass([
  registerGObjectClass
], SelectionTilePreview);

// src/components/tilingsystem/resizeManager.ts
const Meta5 = imports.gi.Meta;
const St7 = imports.gi.St;
const Shell = imports.gi.Shell;
const Clutter9 = imports.gi.Clutter;
var debug8 = logger(`ResizingManager`);
var WINDOW_CLONE_RESIZE_ANIMATION_TIME = 150;
var APP_ICON_SIZE = 96;
var WindowClone = class extends St7.Widget {
  _clone;
  //private _blurWidget: St.Widget;
  constructor(window) {
    super({ layoutManager: new Clutter9.BinLayout(), styleClass: "custom-tile-preview" });
    global.windowGroup.add_child(this);
    this._clone = this._createWindowClone(window);
    this.add_child(this._clone);
    const sigma = 36;
    this._clone.add_effect_with_name("blur", new Shell.BlurEffect({
      //@ts-ignore
      sigma,
      //radius: sigma * 2,
      brightness: 1,
      mode: Shell.BlurMode.ACTOR
      // blur the widget
    }));
    const box = new St7.BoxLayout({
      xAlign: Clutter9.ActorAlign.CENTER,
      yAlign: Clutter9.ActorAlign.CENTER,
      xExpand: true,
      yExpand: true,
      vertical: true,
      style: "spacing: 16px;"
    });
    box.add_child(this._createAppIcon(window, APP_ICON_SIZE));
    box.add_child(new St7.Label({
      xAlign: Clutter9.ActorAlign.CENTER,
      yAlign: Clutter9.ActorAlign.CENTER,
      text: window.get_title(),
      style: "color: white;"
    }));
    this.add_child(box);
    const windowRect = window.get_frame_rect();
    this.set_position(windowRect.x, windowRect.y);
    this.set_size(windowRect.width, windowRect.height);
  }
  _createWindowClone(window) {
    const actor = window.get_compositor_private();
    let actorContent = actor.paint_to_content(window.get_frame_rect());
    let actorClone = new St7.Widget({
      content: actorContent,
      width: window.get_frame_rect().width,
      height: window.get_frame_rect().height,
      xExpand: true,
      yExpand: true
    });
    actorClone.set_offscreen_redirect(Clutter9.OffscreenRedirect.ALWAYS);
    return actorClone;
  }
  _createAppIcon(window, size) {
    let tracker = Shell.WindowTracker.get_default();
    const app = tracker.get_window_app(window);
    let appIcon = app ? app.create_icon_texture(size) : new St7.Icon({ iconName: "application-x-executable", iconSize: size });
    appIcon.xExpand = appIcon.yExpand = true;
    appIcon.xAlign = appIcon.yAlign = Clutter9.ActorAlign.CENTER;
    return appIcon;
  }
};
WindowClone = __decorateClass([
  registerGObjectClass
], WindowClone);
var ResizingManager = class {
  _signals;
  //private _windowToClone: Map<Meta.Window, WindowClone>;
  constructor() {
    this._signals = new SignalHandling();
  }
  destroy() {
    this._signals.disconnect();
  }
  /** From Gnome Shell: https://gitlab.gnome.org/GNOME/gnome-shell/-/blob/main/js/ui/altTab.js#L53
   */
  _getWindows() {
    const workspace = global.workspaceManager.get_active_workspace();
    let windows = global.display.get_tab_list(Meta5.TabList.NORMAL_ALL, workspace);
    return windows.map((w) => {
      return w.is_attached_dialog() ? w.get_transient_for() : w;
    }).filter((w, i, a) => w !== null && !w.skipTaskbar && a.indexOf(w) === i);
  }
  onWindowResizingBegin(window, grabOp) {
    if (!Settings.get_resize_complementing_windows())
      return;
    if (!window || !window.isTiled)
      return;
    const verticalSide = [false, 0];
    const horizontalSide = [false, 0];
    switch (grabOp) {
      case Meta5.GrabOp.RESIZING_N:
      case Meta5.GrabOp.RESIZING_NE:
      case Meta5.GrabOp.RESIZING_NW:
        verticalSide[0] = true;
        verticalSide[1] = St7.Side.TOP;
        break;
      case Meta5.GrabOp.RESIZING_S:
      case Meta5.GrabOp.RESIZING_SE:
      case Meta5.GrabOp.RESIZING_SW:
        verticalSide[0] = true;
        verticalSide[1] = St7.Side.BOTTOM;
        break;
    }
    switch (grabOp) {
      case Meta5.GrabOp.RESIZING_E:
      case Meta5.GrabOp.RESIZING_NE:
      case Meta5.GrabOp.RESIZING_SE:
        horizontalSide[0] = true;
        horizontalSide[1] = St7.Side.RIGHT;
        break;
      case Meta5.GrabOp.RESIZING_W:
      case Meta5.GrabOp.RESIZING_NW:
      case Meta5.GrabOp.RESIZING_SW:
        horizontalSide[0] = true;
        horizontalSide[1] = St7.Side.LEFT;
        break;
    }
    if (!verticalSide[0] && !horizontalSide[0])
      return;
    const otherTiledWindows = this._getWindows().filter((otherWindow) => otherWindow && otherWindow.isTiled && otherWindow !== window && !otherWindow.minimized);
    if (otherTiledWindows.length === 0)
      return;
    const verticalAdjacentWindows = verticalSide[0] ? this._findAdjacent(window, verticalSide[1], new Set(otherTiledWindows)) : [];
    const horizontalAdjacentWindows = horizontalSide[0] ? this._findAdjacent(window, horizontalSide[1], new Set(otherTiledWindows)) : [];
    const windowsMap = /* @__PURE__ */ new Map();
    verticalAdjacentWindows.forEach(([otherWin, sideOtherWin]) => {
      windowsMap.set(otherWin, [
        otherWin,
        otherWin.get_frame_rect().copy(),
        sideOtherWin,
        // resize vertically
        -1
        // resize horizontally
      ]);
    });
    horizontalAdjacentWindows.forEach(([otherWin, sideOtherWin]) => {
      const val = windowsMap.get(otherWin);
      if (val) {
        val[3] = sideOtherWin;
      } else {
        windowsMap.set(otherWin, [
          otherWin,
          otherWin.get_frame_rect().copy(),
          -1,
          // resize vertically
          sideOtherWin
          // resize horizontally
        ]);
      }
    });
    const windowsToResize = Array.from(windowsMap.values());
    this._signals.connect(
      window,
      "size-changed",
      this._onResizingWindow.bind(this, window, window.get_frame_rect().copy(), verticalSide[1], horizontalSide[1], windowsToResize)
    );
  }
  /*private _createWindowClone(window: Meta.Window) {
          if (this._windowToClone.has(window)) return;
  
          const windowClone = new WindowClone(window);
          this._windowToClone.set(window, windowClone);
          windowClone.set_opacity(0);
          windowClone.hide();
      }*/
  _oppositeSide(side) {
    switch (side) {
      case St7.Side.TOP:
        return St7.Side.BOTTOM;
      case St7.Side.BOTTOM:
        return St7.Side.TOP;
      case St7.Side.LEFT:
        return St7.Side.RIGHT;
      case St7.Side.RIGHT:
        return St7.Side.LEFT;
    }
  }
  _findAdjacent(window, side, remainingWindows) {
    const result = [];
    const adjacentWindows = [];
    const windowRect = window.get_frame_rect();
    const borderRect = windowRect.copy();
    const innerGaps = Settings.get_inner_gaps();
    const errorFactor = 4;
    switch (side) {
      case St7.Side.TOP:
        borderRect.height = innerGaps.top + errorFactor;
        borderRect.y -= innerGaps.top + errorFactor;
        break;
      case St7.Side.BOTTOM:
        borderRect.y = borderRect.y + borderRect.height;
        borderRect.height = innerGaps.bottom + errorFactor;
        break;
      case St7.Side.LEFT:
        borderRect.width = innerGaps.left + errorFactor;
        borderRect.x -= innerGaps.left + errorFactor;
        break;
      case St7.Side.RIGHT:
        borderRect.x = borderRect.x + borderRect.width;
        borderRect.width = innerGaps.right + errorFactor;
        break;
    }
    const oppositeSide = this._oppositeSide(side);
    const newRemainingWindows = /* @__PURE__ */ new Set();
    remainingWindows.forEach((otherWin) => {
      const otherWinRect = otherWin.get_frame_rect();
      let [hasIntersection, intersection] = otherWin.get_frame_rect().intersect(borderRect);
      switch (side) {
        case St7.Side.RIGHT:
          hasIntersection && (hasIntersection = intersection.x <= otherWinRect.x);
          break;
        case St7.Side.LEFT:
          hasIntersection && (hasIntersection = intersection.x + intersection.width >= otherWinRect.x + otherWinRect.width);
          break;
        case St7.Side.BOTTOM:
          hasIntersection && (hasIntersection = intersection.y <= otherWinRect.y);
          break;
        case St7.Side.TOP:
          hasIntersection && (hasIntersection = intersection.y + intersection.height >= otherWinRect.y + otherWinRect.height);
          break;
      }
      if (hasIntersection) {
        result.push([otherWin, oppositeSide]);
        adjacentWindows.push(otherWin);
      } else {
        newRemainingWindows.add(otherWin);
      }
    });
    adjacentWindows.forEach((otherWin) => {
      this._findAdjacent(otherWin, oppositeSide, newRemainingWindows).forEach((recursionResult) => {
        result.push(recursionResult);
        newRemainingWindows.delete(recursionResult[0]);
      });
    });
    return result;
  }
  onWindowResizingEnd(window) {
    this._signals.disconnect(window);
  }
  _onResizingWindow(window, startingRect, resizeVerticalSide, resizeHorizontalSide, windowsToResize) {
    const currentRect = window.get_frame_rect();
    const resizedRect = {
      x: currentRect.x - startingRect.x,
      y: currentRect.y - startingRect.y,
      width: currentRect.width - startingRect.width,
      height: currentRect.height - startingRect.height
    };
    windowsToResize.forEach(([otherWindow, otherWindowRect, verticalSide, horizontalSide]) => {
      const isSameVerticalSide = verticalSide !== -1 && verticalSide === resizeVerticalSide;
      const isSameHorizontalSide = horizontalSide !== -1 && horizontalSide === resizeHorizontalSide;
      const rect = [otherWindowRect.x, otherWindowRect.y, otherWindowRect.width, otherWindowRect.height];
      if (horizontalSide === St7.Side.LEFT) {
        rect[2] = otherWindowRect.width - (isSameHorizontalSide ? resizedRect.x : resizedRect.width);
        rect[0] = otherWindowRect.x + (isSameHorizontalSide ? resizedRect.x : resizedRect.width);
      } else if (horizontalSide === St7.Side.RIGHT) {
        rect[2] = otherWindowRect.width + (isSameHorizontalSide ? resizedRect.width : resizedRect.x);
      }
      if (verticalSide === St7.Side.TOP) {
        rect[3] = otherWindowRect.height - (isSameVerticalSide ? resizedRect.y : resizedRect.height);
        rect[1] = otherWindowRect.y + (isSameVerticalSide ? resizedRect.y : resizedRect.height);
      } else if (verticalSide === St7.Side.BOTTOM) {
        rect[3] = otherWindowRect.height + (isSameVerticalSide ? resizedRect.height : resizedRect.y);
      }
      otherWindow.move_resize_frame(
        false,
        Math.max(0, rect[0]),
        Math.max(0, rect[1]),
        Math.max(0, rect[2]),
        Math.max(0, rect[3])
      );
    });
  }
};

// src/components/tilingsystem/tilingManager.ts
const Meta6 = imports.gi.Meta;
const Main3 = imports.ui.main;
const Clutter10 = imports.gi.Clutter;
const GLib = imports.gi.GLib;
var TilingManager = class {
  _monitor;
  _selectedTilesPreview;
  _snapAssist;
  _tilingLayout;
  _resizingManager;
  _workArea;
  _innerGaps;
  _outerGaps;
  _enableScaling;
  _isGrabbingWindow;
  _movingWindowTimerDuration = 15;
  _lastCursorPos = null;
  _wasAltPressed;
  _wasCtrlPressed;
  _isSnapAssisting;
  _movingWindowTimerId = null;
  _signals;
  _debug;
  /**
   * Constructs a new TilingManager instance.
   * @param monitor The monitor to manage tiling for.
   */
  constructor(monitor, enableScaling) {
    this._isGrabbingWindow = false;
    this._wasAltPressed = false;
    this._wasCtrlPressed = false;
    this._isSnapAssisting = false;
    this._enableScaling = enableScaling;
    this._monitor = monitor;
    this._signals = new SignalHandling();
    this._debug = logger(`TilingManager ${monitor.index}`);
    const layout = GlobalState.get().getSelectedLayoutOfMonitor(monitor.index);
    this._innerGaps = buildMargin(Settings.get_inner_gaps());
    this._outerGaps = buildMargin(Settings.get_outer_gaps());
    this._workArea = Main3.layoutManager.getWorkAreaForMonitor(this._monitor.index);
    this._debug(`Work area for monitor ${this._monitor.index}: ${this._workArea.x} ${this._workArea.y} ${this._workArea.width}x${this._workArea.height}`);
    const monitorScalingFactor = this._enableScaling ? getScalingFactor(monitor.index) : void 0;
    this._tilingLayout = new TilingLayout(layout, this._innerGaps, this._outerGaps, this._workArea, monitorScalingFactor);
    this._selectedTilesPreview = new SelectionTilePreview({ parent: global.windowGroup });
    this._snapAssist = new SnapAssist(global.windowGroup, this._workArea, monitorScalingFactor);
    this._resizingManager = new ResizingManager();
  }
  /**
   * Enables tiling manager by setting up event listeners:
   *  - handle any window's grab begin.
   *  - handle any window's grab end.
   *  - handle grabbed window's movement.
   */
  enable() {
    this._signals.connect(Settings, Settings.SETTING_SELECTED_LAYOUTS, () => {
      const layout = GlobalState.get().getSelectedLayoutOfMonitor(this._monitor.index);
      this._tilingLayout.relayout({ layout });
    });
    this._signals.connect(GlobalState.get(), GlobalState.SIGNAL_LAYOUTS_CHANGED, () => {
      const layout = GlobalState.get().getSelectedLayoutOfMonitor(this._monitor.index);
      this._tilingLayout.relayout({ layout });
    });
    this._signals.connect(Settings, Settings.SETTING_INNER_GAPS, () => {
      this._innerGaps = buildMargin(Settings.get_inner_gaps());
      this._tilingLayout.relayout({ innerGaps: this._innerGaps });
    });
    this._signals.connect(Settings, Settings.SETTING_OUTER_GAPS, () => {
      this._outerGaps = buildMargin(Settings.get_outer_gaps());
      this._tilingLayout.relayout({ outerGaps: this._outerGaps });
    });
    this._signals.connect(global.display, "grab-op-begin", (_display, window, grabOp) => {
      if (grabOp != Meta6.GrabOp.MOVING) {
        this._resizingManager.onWindowResizingBegin(window, grabOp);
        return;
      }
      this._onWindowGrabBegin(window);
    });
    this._signals.connect(global.display, "grab-op-end", (_display, window, grabOp) => {
      if (!this._isGrabbingWindow) {
        this._resizingManager.onWindowResizingEnd(window);
        return;
      }
      this._onWindowGrabEnd(window);
    });
    this._signals.connect(
      this._snapAssist,
      "snap-assist",
      (_, tile) => this._onSnapAssist(tile)
    );
  }
  /**
   * Destroys the tiling manager and cleans up resources.
   */
  destroy() {
    if (this._movingWindowTimerId) {
      GLib.Source.remove(this._movingWindowTimerId);
      this._movingWindowTimerId = null;
    }
    this._signals.disconnect();
    this._isGrabbingWindow = false;
    this._isSnapAssisting = false;
    this._tilingLayout.destroy();
    this._snapAssist.destroy();
    this._selectedTilesPreview.destroy();
    this._resizingManager.destroy();
  }
  set workArea(newWorkArea) {
    if (newWorkArea.equal(this._workArea))
      return;
    this._workArea = newWorkArea;
    this._debug(`new work area for monitor ${this._monitor.index}: ${newWorkArea.x} ${newWorkArea.y} ${newWorkArea.width}x${newWorkArea.height}`);
    this._tilingLayout.relayout({ containerRect: this._workArea });
    this._snapAssist.workArea = this._workArea;
  }
  _onWindowGrabBegin(window) {
    if (this._isGrabbingWindow)
      return;
    this._isGrabbingWindow = true;
    this._movingWindowTimerId = GLib.timeout_add(
      GLib.PRIORITY_DEFAULT_IDLE,
      this._movingWindowTimerDuration,
      this._onMovingWindow.bind(this, window)
    );
    this._onMovingWindow(window);
  }
  _onMovingWindow(window) {
    if (!this._isGrabbingWindow) {
      this._movingWindowTimerId = null;
      return GLib.SOURCE_REMOVE;
    }
    if (!window.allows_resize() || !window.allows_move() || !this._isPointerInsideThisMonitor()) {
      this._tilingLayout.close();
      this._selectedTilesPreview.close();
      this._snapAssist.close(true);
      this._isSnapAssisting = false;
      return GLib.SOURCE_CONTINUE;
    }
    const extWin = window;
    extWin.isTiled = false;
    if (extWin.originalSize) {
      const newSize = buildRectangle({
        x: window.get_frame_rect().x,
        y: window.get_frame_rect().y,
        width: extWin.originalSize.width,
        height: extWin.originalSize.height
      });
      if (Settings.get_restore_window_original_size()) {
        this._easeWindowRect(window, newSize);
      }
      extWin.originalSize = void 0;
    }
    const [x, y, modifier] = global.get_pointer();
    const currPointerPos = { x, y };
    const isAltPressed = (modifier & Clutter10.ModifierType.MOD1_MASK) != 0;
    const isCtrlPressed = (modifier & Clutter10.ModifierType.CONTROL_MASK) != 0;
    const allowSpanMultipleTiles = Settings.get_span_multiple_tiles() && isAltPressed;
    const showTilingSystem = Settings.get_tiling_system_enabled() && isCtrlPressed;
    const changedSpanMultipleTiles = Settings.get_span_multiple_tiles() && isAltPressed !== this._wasAltPressed;
    const changedShowTilingSystem = Settings.get_tiling_system_enabled() && isCtrlPressed !== this._wasCtrlPressed;
    if (!changedSpanMultipleTiles && !changedShowTilingSystem && currPointerPos.x === this._lastCursorPos?.x && currPointerPos.y === this._lastCursorPos?.y) {
      return GLib.SOURCE_CONTINUE;
    }
    this._lastCursorPos = currPointerPos;
    this._wasCtrlPressed = isCtrlPressed;
    this._wasAltPressed = isAltPressed;
    if (!showTilingSystem) {
      if (this._tilingLayout.showing) {
        this._tilingLayout.close();
        this._selectedTilesPreview.close();
      }
      if (Settings.get_snap_assist_enabled()) {
        this._snapAssist.onMovingWindow(window, true, currPointerPos);
      }
      return GLib.SOURCE_CONTINUE;
    }
    if (!this._tilingLayout.showing) {
      this._tilingLayout.openAbove(window);
      this._snapAssist.close(true);
    }
    if (this._isSnapAssisting) {
      this._selectedTilesPreview.close();
      this._isSnapAssisting = false;
    }
    if (!changedSpanMultipleTiles && isPointInsideRect(currPointerPos, this._selectedTilesPreview.rect)) {
      return GLib.SOURCE_CONTINUE;
    }
    let selectionRect = this._tilingLayout.getTileBelow(currPointerPos, changedSpanMultipleTiles && !allowSpanMultipleTiles);
    if (!selectionRect)
      return GLib.SOURCE_CONTINUE;
    selectionRect = selectionRect.copy();
    if (allowSpanMultipleTiles) {
      selectionRect = selectionRect.union(this._selectedTilesPreview.rect);
    }
    this._tilingLayout.hoverTilesInRect(selectionRect, !allowSpanMultipleTiles);
    this._selectedTilesPreview.gaps = buildTileGaps(
      selectionRect,
      this._tilingLayout.innerGaps,
      this._tilingLayout.outerGaps,
      this._workArea,
      this._enableScaling ? getScalingFactorOf(this._tilingLayout)[1] : void 0
    );
    this._selectedTilesPreview.openAbove(
      window,
      true,
      selectionRect
    );
    return GLib.SOURCE_CONTINUE;
  }
  _onWindowGrabEnd(window) {
    this._isGrabbingWindow = false;
    this._tilingLayout.close();
    const selectionRect = buildRectangle({
      x: this._selectedTilesPreview.innerX,
      y: this._selectedTilesPreview.innerY,
      width: this._selectedTilesPreview.innerWidth,
      height: this._selectedTilesPreview.innerHeight
    });
    this._selectedTilesPreview.close();
    this._snapAssist.close(true);
    this._lastCursorPos = null;
    const isCtrlPressed = global.get_pointer()[2] & Clutter10.ModifierType.CONTROL_MASK;
    if (!isCtrlPressed && !this._isSnapAssisting)
      return;
    this._isSnapAssisting = false;
    if (!this._isPointerInsideThisMonitor())
      return;
    if (selectionRect.width <= 0 || selectionRect.height <= 0) {
      return;
    }
    window.originalSize = window.get_frame_rect().copy();
    window.isTiled = true;
    this._easeWindowRect(window, selectionRect);
  }
  _easeWindowRect(window, destRect) {
    const windowActor = window.get_compositor_private();
    windowActor.remove_all_transitions();
    Main3.wm._prepareAnimationInfo(
      global.windowManager,
      windowActor,
      window.get_frame_rect().copy(),
      Meta6.SizeChange.UNMAXIMIZE
    );
    window.move_to_monitor(this._monitor.index);
    window.move_resize_frame(
      false,
      destRect.x,
      destRect.y,
      destRect.width,
      destRect.height
    );
  }
  _onSnapAssist(tile) {
    if (tile.width === 0 || tile.height === 0) {
      this._selectedTilesPreview.close();
      this._isSnapAssisting = false;
      return;
    }
    const scaledRect = TileUtils.apply_props(tile, this._workArea);
    if (scaledRect.x + scaledRect.width > this._workArea.x + this._workArea.width) {
      scaledRect.width -= scaledRect.x + scaledRect.width - this._workArea.x - this._workArea.width;
    }
    if (scaledRect.y + scaledRect.height > this._workArea.y + this._workArea.height) {
      scaledRect.height -= scaledRect.y + scaledRect.height - this._workArea.y - this._workArea.height;
    }
    this._selectedTilesPreview.gaps = buildTileGaps(
      scaledRect,
      this._tilingLayout.innerGaps,
      this._tilingLayout.outerGaps,
      this._workArea,
      this._enableScaling ? getScalingFactorOf(this._tilingLayout)[1] : void 0
    );
    this._selectedTilesPreview.open(true, scaledRect);
    this._isSnapAssisting = true;
  }
  /**
   * Checks if pointer is inside the current monitor
   * @returns true if the pointer is inside the current monitor, false otherwise
   */
  _isPointerInsideThisMonitor() {
    const [x, y] = global.get_pointer();
    return x >= this._monitor.x && x <= this._monitor.x + this._monitor.width && y >= this._monitor.y && y <= this._monitor.y + this._monitor.height;
  }
};

// src/components/editor/editableTilePreview.ts
const St8 = imports.gi.St;
const Mtk11 = Mtk;
var debug9 = logger("EditableTilePreview");
var EditableTilePreview = class extends TilePreview {
  _btn;
  _tile;
  _containerRect;
  _sliders;
  _signals;
  constructor(params) {
    super(params);
    this.add_style_class_name("editable-tile-preview");
    this._tile = params.tile;
    this._containerRect = params.containerRect;
    this._sliders = [null, null, null, null];
    this._signals = [null, null, null, null];
    this._btn = new St8.Button({
      styleClass: "editable-tile-preview-button",
      xExpand: true,
      trackHover: true
    });
    this.add_child(this._btn);
    this._btn.set_size(this.innerWidth, this.innerHeight);
    this._btn.set_button_mask(St8.ButtonMask.ONE | St8.ButtonMask.THREE);
    this._updateLabelText();
    this.connect("destroy", this._onDestroy.bind(this));
  }
  get tile() {
    return this._tile;
  }
  getSlider(side) {
    return this._sliders[side];
  }
  getAllSliders() {
    return [...this._sliders];
  }
  get hover() {
    return this._btn.hover;
  }
  addSlider(slider, side) {
    const sig = this._signals[side];
    if (sig)
      this._sliders[side]?.disconnect(sig);
    this._sliders[side] = slider;
    this._signals[side] = slider.connect("slide", () => this._onSliderMove(side));
    this._tile.groups = [];
    this._sliders.forEach((sl) => sl && this._tile.groups.push(sl.groupId));
  }
  removeSlider(side) {
    if (this._sliders[side] === null)
      return;
    const sig = this._signals[side];
    if (sig)
      this._sliders[side]?.disconnect(sig);
    this._sliders[side] = null;
    this._tile.groups = [];
    this._sliders.forEach((sl) => sl && this._tile.groups.push(sl.groupId));
  }
  updateTile({ x, y, width, height }) {
    const oldSize = this._rect.copy();
    this._tile.x = x;
    this._tile.y = y;
    this._tile.width = width;
    this._tile.height = height;
    this._rect = TileUtils.apply_props(this._tile, this._containerRect);
    this.set_size(this.innerWidth, this.innerHeight);
    this.set_position(this.innerX, this.innerY);
    this._btn.set_size(this.width, this.height);
    this._updateLabelText();
    const newSize = this._rect.copy();
    this.emit("size-changed", oldSize, newSize);
  }
  connect(signal, callback) {
    if (signal === "clicked" || signal === "notify::hover" || signal === "motion-event") {
      return this._btn.connect(signal, callback);
    }
    return super.connect(signal, callback);
  }
  _updateLabelText() {
    this._btn.label = `${this.innerWidth}x${this.innerHeight}`;
  }
  _onSliderMove(side) {
    const slider = this._sliders[side];
    if (slider === null)
      return;
    const posHoriz = (slider.x + slider.width / 2 - this._containerRect.x) / this._containerRect.width;
    const posVert = (slider.y + slider.height / 2 - this._containerRect.y) / this._containerRect.height;
    switch (side) {
      case St8.Side.TOP:
        this._tile.height += this._tile.y - posVert;
        this._tile.y = posVert;
        break;
      case St8.Side.RIGHT:
        this._tile.width = posHoriz - this._tile.x;
        break;
      case St8.Side.BOTTOM:
        this._tile.height = posVert - this._tile.y;
        break;
      case St8.Side.LEFT:
        this._tile.width += this._tile.x - posHoriz;
        this._tile.x = posHoriz;
        break;
    }
    this.updateTile({ ...this._tile });
  }
  _onDestroy() {
    this._signals.forEach((id, side) => id && this._sliders[side]?.disconnect(id));
  }
};
__publicField(EditableTilePreview, "metaInfo", {
  Signals: {
    "size-changed": {
      //@ts-ignore todo
      param_types: [Mtk11.Rectangle.$gtype, Mtk11.Rectangle.$gtype]
      // oldSize, newSize
    }
  },
  GTypeName: "EditableTilePreview"
});
__publicField(EditableTilePreview, "MIN_TILE_SIZE", 140);
EditableTilePreview = __decorateClass([
  registerGObjectClass
], EditableTilePreview);

// src/components/editor/slider.ts
const Clutter12 = imports.gi.Clutter;
const St9 = imports.gi.St;
const Meta7 = imports.gi.Meta;
const GObject5 = imports.gi.GObject;
var debug10 = logger("Slider");
var Slider2 = class extends St9.Button {
  _sliderSize = 48;
  _groupId;
  _signals;
  _dragging;
  _grab;
  _horizontalDir;
  _lastEventCoord;
  _previousTiles;
  _nextTiles;
  _minTileCoord;
  _maxTileCoord;
  _scalingFactor;
  constructor(parent, groupId, x, y, horizontal) {
    super({
      styleClass: "layout-editor-slider",
      canFocus: true,
      xExpand: false,
      trackHover: true
    });
    parent.add_child(this);
    this._signals = /* @__PURE__ */ new Map();
    this._groupId = groupId;
    this._horizontalDir = horizontal;
    const [_, scalingFactor] = getScalingFactorOf(this);
    this._scalingFactor = scalingFactor;
    this.set_width(this.desiredWidth);
    this.set_height(this.desiredHeight);
    this._previousTiles = [];
    this._nextTiles = [];
    this._minTileCoord = Number.MAX_VALUE;
    this._maxTileCoord = Number.MIN_VALUE;
    this._dragging = false;
    this._lastEventCoord = null;
    this.set_position(Math.round(Math.round(x - this.width / 2)), Math.round(y - this.height / 2));
    this.connect("notify::hover", () => global.display.set_cursor(this.preferredCursor));
    this.connect("destroy", this._onDestroy.bind(this));
  }
  get groupId() {
    return this._groupId;
  }
  get desiredWidth() {
    return (this._horizontalDir ? 12 : this._sliderSize) * this._scalingFactor;
  }
  get desiredHeight() {
    return (this._horizontalDir ? this._sliderSize : 12) * this._scalingFactor;
  }
  get preferredCursor() {
    return this.hover || this._dragging ? this._horizontalDir ? Meta7.Cursor.WEST_RESIZE : Meta7.Cursor.NORTH_RESIZE : Meta7.Cursor.DEFAULT;
  }
  addTile(tile) {
    const isNext = this._horizontalDir ? this.x <= tile.rect.x : this.y <= tile.rect.y;
    if (isNext)
      this._nextTiles.push(tile);
    else
      this._previousTiles.push(tile);
    const side = this._horizontalDir ? isNext ? St9.Side.LEFT : St9.Side.RIGHT : isNext ? St9.Side.TOP : St9.Side.BOTTOM;
    tile.addSlider(this, side);
    this._minTileCoord = Math.min(this._minTileCoord, this._horizontalDir ? tile.rect.y : tile.rect.x);
    this._maxTileCoord = Math.max(this._maxTileCoord, this._horizontalDir ? tile.rect.y + tile.rect.height : tile.rect.x + tile.rect.width);
    this._updatePosition();
    this._createTileSignals(tile);
  }
  _onTileSizeChanged(tile, oldSize, newSize) {
    if (this._horizontalDir) {
      if (this._minTileCoord !== oldSize.y && this._maxTileCoord !== oldSize.y + oldSize.height) {
        return;
      }
      if (this._minTileCoord === oldSize.y) {
        this._minTileCoord = newSize.y;
      }
      if (this._maxTileCoord === oldSize.y + oldSize.height) {
        this._maxTileCoord = newSize.y + newSize.height;
      }
    } else {
      if (this._minTileCoord !== oldSize.x && this._maxTileCoord !== oldSize.x + oldSize.width) {
        return;
      }
      if (this._minTileCoord === oldSize.x) {
        this._minTileCoord = newSize.x;
      }
      if (this._maxTileCoord === oldSize.x + oldSize.width) {
        this._maxTileCoord = newSize.x + newSize.width;
      }
    }
    this._updatePosition();
  }
  _updatePosition() {
    this.set_width(this.desiredWidth);
    this.set_height(this.desiredHeight);
    const newCoord = (this._minTileCoord + this._maxTileCoord) / 2;
    if (this._horizontalDir)
      this.set_y(Math.round(newCoord - this.height / 2));
    else
      this.set_x(Math.round(newCoord - this.width / 2));
  }
  _onTileDeleted(tile) {
    const isNext = this._horizontalDir ? this.x <= tile.rect.x : this.y <= tile.rect.y;
    const array = isNext ? this._nextTiles : this._previousTiles;
    const index = array.indexOf(tile, 0);
    if (index >= 0) {
      array.splice(index, 1);
    }
    const sig = this._signals.get(tile);
    if (sig) {
      sig.forEach((id) => tile.disconnect(id));
      this._signals.delete(tile);
    }
  }
  onTileSplit(tileToRemove, newTiles) {
    if (newTiles.length === 0)
      return;
    const isNext = this._horizontalDir ? this.x <= tileToRemove.rect.x : this.y <= tileToRemove.rect.y;
    const array = isNext ? this._nextTiles : this._previousTiles;
    const index = array.indexOf(tileToRemove);
    if (index < 0)
      return;
    const side = this._horizontalDir ? isNext ? St9.Side.LEFT : St9.Side.RIGHT : isNext ? St9.Side.TOP : St9.Side.BOTTOM;
    const sig = this._signals.get(tileToRemove);
    if (sig) {
      sig.forEach((id) => tileToRemove.disconnect(id));
      this._signals.delete(tileToRemove);
    }
    array[index] = newTiles[0];
    newTiles[0].addSlider(this, side);
    this._createTileSignals(newTiles[0]);
    for (let i = 1; i < newTiles.length; i++) {
      const tile = newTiles[i];
      array.push(tile);
      tile.addSlider(this, side);
      this._createTileSignals(tile);
    }
  }
  _createTileSignals(tile) {
    if (this._signals.has(tile))
      return;
    this._signals.set(tile, []);
    this._signals.get(tile)?.push(
      tile.connect("size-changed", this._onTileSizeChanged.bind(this))
    );
    this._signals.get(tile)?.push(
      tile.connect("destroy", this._onTileDeleted.bind(this))
    );
  }
  deleteSlider(tileToDelete) {
    const isNext = this._horizontalDir ? this.x <= tileToDelete.rect.x : this.y <= tileToDelete.rect.y;
    const array = isNext ? this._nextTiles : this._previousTiles;
    if (array.length > 1 || array[0] !== tileToDelete)
      return false;
    array.pop();
    const oppositeSide = this._horizontalDir ? isNext ? St9.Side.RIGHT : St9.Side.LEFT : isNext ? St9.Side.BOTTOM : St9.Side.TOP;
    (isNext ? this._previousTiles : this._nextTiles).forEach((tileToExtend) => {
      tileToExtend.updateTile({
        x: !isNext && this._horizontalDir ? tileToDelete.tile.x : tileToExtend.tile.x,
        y: !isNext && !this._horizontalDir ? tileToDelete.tile.y : tileToExtend.tile.y,
        width: this._horizontalDir ? tileToExtend.tile.width + tileToDelete.tile.width : tileToExtend.tile.width,
        height: this._horizontalDir ? tileToExtend.tile.height : tileToExtend.tile.height + tileToDelete.tile.height
      });
      tileToExtend.removeSlider(oppositeSide);
      tileToDelete.getSlider(oppositeSide)?.addTile(tileToExtend);
    });
    return true;
  }
  vfunc_button_press_event(event) {
    return this._startDragging(event);
  }
  vfunc_button_release_event() {
    if (this._dragging)
      return this._endDragging();
    return Clutter12.EVENT_PROPAGATE;
  }
  vfunc_motion_event(event) {
    if (this._dragging) {
      const [stageX, stageY] = getEventCoords(event);
      this._move(stageX, stageY);
      return Clutter12.EVENT_STOP;
    }
    return Clutter12.EVENT_PROPAGATE;
  }
  _startDragging(event) {
    if (this._dragging)
      return Clutter12.EVENT_PROPAGATE;
    this._dragging = true;
    global.display.set_cursor(this.preferredCursor);
    this._grab = global.stage.grab(this);
    const [stageX, stageY] = getEventCoords(event);
    this._move(stageX, stageY);
    return Clutter12.EVENT_STOP;
  }
  _endDragging() {
    if (this._dragging) {
      if (this._grab) {
        this._grab.dismiss();
        this._grab = null;
      }
      this._dragging = false;
      this._lastEventCoord = null;
    }
    global.display.set_cursor(this.preferredCursor);
    return Clutter12.EVENT_STOP;
  }
  _move(eventX, eventY) {
    eventX = Math.round(eventX);
    eventY = Math.round(eventY);
    if (this._lastEventCoord !== null) {
      const movement = {
        x: this._horizontalDir ? eventX - this._lastEventCoord.x : 0,
        y: this._horizontalDir ? 0 : eventY - this._lastEventCoord.y
      };
      for (const prevTile of this._previousTiles) {
        if (prevTile.rect.width + movement.x < EditableTilePreview.MIN_TILE_SIZE || prevTile.rect.height + movement.y < EditableTilePreview.MIN_TILE_SIZE)
          return;
      }
      for (const nextTile of this._nextTiles) {
        if (nextTile.rect.width - movement.x < EditableTilePreview.MIN_TILE_SIZE || nextTile.rect.height - movement.y < EditableTilePreview.MIN_TILE_SIZE)
          return;
      }
      this.set_position(this.x + movement.x, this.y + movement.y);
      this.emit("slide", this._horizontalDir ? movement.x : movement.y);
    }
    this._lastEventCoord = { x: eventX, y: eventY };
  }
  _onDestroy() {
    this._signals.forEach((ids, tile) => ids.forEach((id) => tile.disconnect(id)));
    this._minTileCoord = Number.MAX_VALUE;
    this._maxTileCoord = Number.MIN_VALUE;
    this._previousTiles = [];
    this._nextTiles = [];
    this._lastEventCoord = null;
    this._endDragging();
  }
};
__publicField(Slider2, "metaInfo", {
  Signals: {
    "slide": {
      param_types: [GObject5.TYPE_INT]
      // movement
    }
  },
  GTypeName: "Slider"
});
Slider2 = __decorateClass([
  registerGObjectClass
], Slider2);

// src/components/editor/hoverLine.ts
const St10 = imports.gi.St;
const GLib2 = imports.gi.GLib;
const Shell2 = imports.gi.Shell;
const Clutter13 = imports.gi.Clutter;
var debug11 = logger("HoverLine");
var HoverLine = class extends St10.Widget {
  _hoverTimer;
  _size;
  _workArea;
  _hoveredTile;
  constructor(parent, workArea) {
    super({ styleClass: "hover-line" });
    parent.add_child(this);
    this._hoveredTile = null;
    const [_, scalingFactor] = getScalingFactorOf(this);
    this._size = 16 * scalingFactor;
    this._workArea = workArea;
    this.hide();
    this._hoverTimer = GLib2.timeout_add(
      GLib2.PRIORITY_DEFAULT_IDLE,
      100,
      this._handleModifierChange.bind(this)
    );
    this.connect("destroy", this._onDestroy.bind(this));
  }
  handleTileDestroy(tile) {
    if (this._hoveredTile === tile) {
      this._hoveredTile = null;
      this.hide();
    }
  }
  handleMouseMove(tile, x, y) {
    this._hoveredTile = tile;
    const modifier = Shell2.Global.get().get_pointer()[2];
    const splitHorizontally = (modifier & Clutter13.ModifierType.CONTROL_MASK) == 0;
    this._drawLine(splitHorizontally, x, y);
  }
  _handleModifierChange() {
    if (!this._hoveredTile)
      return GLib2.SOURCE_CONTINUE;
    if (!this._hoveredTile.hover) {
      this.hide();
      return GLib2.SOURCE_CONTINUE;
    }
    const [x, y, modifier] = global.get_pointer();
    const splitHorizontally = (modifier & Clutter13.ModifierType.CONTROL_MASK) == 0;
    this._drawLine(splitHorizontally, x, y);
    return GLib2.SOURCE_CONTINUE;
  }
  _drawLine(splitHorizontally, x, y) {
    if (!this._hoveredTile)
      return;
    if (splitHorizontally) {
      const newX = x - this._size / 2 - this._workArea.x;
      if (newX < this._hoveredTile.x || newX + this._size > this._hoveredTile.x + this._hoveredTile.width) {
        return;
      }
      this.set_size(this._size, this._hoveredTile.height);
      this.set_position(newX, this._hoveredTile.y);
    } else {
      const newY = y - this._size / 2 - this._workArea.y;
      if (newY < this._hoveredTile.y || newY + this._size > this._hoveredTile.y + this._hoveredTile.height) {
        return;
      }
      this.set_size(this._hoveredTile.width, this._size);
      this.set_position(this._hoveredTile.x, newY);
    }
    this.show();
  }
  _onDestroy() {
    GLib2.Source.remove(this._hoverTimer);
  }
};
HoverLine = __decorateClass([
  registerGObjectClass
], HoverLine);

// src/components/editor/layoutEditor.ts
const Clutter14 = imports.gi.Clutter;
const St11 = imports.gi.St;
const Shell3 = imports.gi.Shell;
const Main4 = imports.ui.main;
var debug12 = logger("LayoutEditor");
var LayoutEditor = class extends St11.Widget {
  _layout;
  _containerRect;
  _sliders;
  _innerGaps;
  _outerGaps;
  _minimizedWindows;
  _hoverWidget;
  constructor(layout, monitor, enableScaling) {
    super({ styleClass: "layout-editor" });
    global.windowGroup.add_child(this);
    if (enableScaling) {
      const scalingFactor = getScalingFactor(monitor.index);
      enableScalingFactorSupport(this, scalingFactor);
    }
    const workArea = Main4.layoutManager.getWorkAreaForMonitor(monitor.index);
    this.set_position(workArea.x, workArea.y);
    this.set_size(workArea.width, workArea.height);
    this._innerGaps = buildMargin(Settings.get_inner_gaps());
    this._outerGaps = buildMargin(Settings.get_outer_gaps());
    this._sliders = [];
    this._containerRect = buildRectangle({ x: 0, y: 0, width: workArea.width, height: workArea.height });
    this._minimizedWindows = getWindowsOfMonitor(monitor).filter((win) => !win.is_hidden());
    this._minimizedWindows.forEach((win) => win.can_minimize() && win.minimize());
    this._hoverWidget = new HoverLine(this, workArea.copy());
    this.connect("destroy", this._onDestroy.bind(this));
    this._layout = layout;
    this._drawEditor();
  }
  get layout() {
    return this._layout;
  }
  set layout(newLayout) {
    this._sliders.forEach((slider) => slider.destroy());
    this._sliders = [];
    this.remove_all_children();
    this._layout = newLayout;
    this._drawEditor();
  }
  _drawEditor() {
    const groups = /* @__PURE__ */ new Map();
    this._layout.tiles.forEach((tile) => {
      const rect = TileUtils.apply_props(tile, this._containerRect);
      const prev = this._buildEditableTile(tile, rect);
      tile.groups.forEach((id) => {
        if (!groups.has(id))
          groups.set(id, []);
        groups.get(id)?.push(prev);
      });
    });
    groups.forEach((tiles, groupdId) => {
      let lines = tiles.flatMap((t) => [
        { c: Math.round(t.tile.x * 1e3) / 1e3, end: false, r: t.rect.x },
        { c: Math.round((t.tile.x + t.tile.width) * 1e3) / 1e3, end: true, r: t.rect.x + t.rect.width }
      ]).sort((a, b) => a.c - b.c !== 0 ? a.c - b.c : a.end ? -1 : 1);
      let count = 0;
      let coord = -1;
      let horizontal = false;
      for (const line of lines) {
        count += line.end ? -1 : 1;
        if (count === 0 && line !== lines[lines.length - 1]) {
          coord = line.r;
          horizontal = true;
          break;
        }
      }
      if (coord === -1) {
        lines = tiles.flatMap((t) => [
          { c: Math.round(t.tile.y * 1e3) / 1e3, end: false, r: t.rect.y },
          { c: Math.round((t.tile.y + t.tile.height) * 1e3) / 1e3, end: true, r: t.rect.y + t.rect.height }
        ]).sort((a, b) => a.c - b.c !== 0 ? a.c - b.c : a.end ? -1 : 1);
        count = 0;
        for (const line of lines) {
          count += line.end ? -1 : 1;
          if (count === 0 && line !== lines[lines.length - 1]) {
            coord = line.r;
            break;
          }
        }
      }
      const slider = this._buildSlider(horizontal, coord, groupdId);
      this._sliders.push(slider);
      tiles.forEach((editable) => slider.addTile(editable));
    });
  }
  _buildEditableTile(tile, rect) {
    const gaps = buildTileGaps(rect, this._innerGaps, this._outerGaps, this._containerRect);
    const editableTile = new EditableTilePreview({ parent: this, tile, containerRect: this._containerRect, rect, gaps });
    editableTile.open();
    editableTile.connect("clicked", (_, clicked_button) => {
      if (clicked_button === St11.ButtonMask.ONE)
        this.splitTile(editableTile);
      else if (clicked_button === 3)
        this.deleteTile(editableTile);
    });
    editableTile.connect("motion-event", (tile2, event) => {
      const [stageX, stageY] = getEventCoords(event);
      this._hoverWidget.handleMouseMove(editableTile, stageX, stageY);
      return Clutter14.EVENT_PROPAGATE;
    });
    editableTile.connect("notify::hover", () => {
      const [stageX, stageY] = Shell3.Global.get().get_pointer();
      this._hoverWidget.handleMouseMove(editableTile, stageX, stageY);
    });
    if (this._sliders.length > 0)
      this.set_child_below_sibling(editableTile, this._sliders[0]);
    return editableTile;
  }
  splitTile(editableTile) {
    const oldTile = editableTile.tile;
    const index = this._layout.tiles.indexOf(oldTile);
    if (index < 0)
      return;
    const [x, y, modifier] = global.get_pointer();
    const splitX = (x - this.x) / this._containerRect.width;
    const splitY = (y - this.y) / this._containerRect.height;
    const splitHorizontally = (modifier & Clutter14.ModifierType.CONTROL_MASK) == 0;
    const prevTile = new Tile2({
      x: oldTile.x,
      y: oldTile.y,
      width: splitHorizontally ? splitX - oldTile.x : oldTile.width,
      height: splitHorizontally ? oldTile.height : splitY - oldTile.y,
      groups: []
    });
    const nextTile = new Tile2({
      x: splitHorizontally ? splitX : oldTile.x,
      y: splitHorizontally ? oldTile.y : splitY,
      width: splitHorizontally ? oldTile.width - prevTile.width : oldTile.width,
      height: splitHorizontally ? oldTile.height : oldTile.height - prevTile.height,
      groups: []
    });
    const prevRect = TileUtils.apply_props(prevTile, this._containerRect);
    const nextRect = TileUtils.apply_props(nextTile, this._containerRect);
    if (prevRect.height < EditableTilePreview.MIN_TILE_SIZE || prevRect.width < EditableTilePreview.MIN_TILE_SIZE || nextRect.height < EditableTilePreview.MIN_TILE_SIZE || nextRect.width < EditableTilePreview.MIN_TILE_SIZE)
      return;
    this._layout.tiles[index] = prevTile;
    this._layout.tiles.push(nextTile);
    const prevEditableTile = this._buildEditableTile(prevTile, prevRect);
    const nextEditableTile = this._buildEditableTile(nextTile, nextRect);
    const slider = this._buildSlider(splitHorizontally, splitHorizontally ? nextEditableTile.rect.x : nextEditableTile.rect.y);
    this._sliders.push(slider);
    slider.addTile(prevEditableTile);
    slider.addTile(nextEditableTile);
    if (splitHorizontally) {
      editableTile.getSlider(St11.Side.TOP)?.onTileSplit(editableTile, [prevEditableTile, nextEditableTile]);
      editableTile.getSlider(St11.Side.BOTTOM)?.onTileSplit(editableTile, [prevEditableTile, nextEditableTile]);
      editableTile.getSlider(St11.Side.LEFT)?.onTileSplit(editableTile, [prevEditableTile]);
      editableTile.getSlider(St11.Side.RIGHT)?.onTileSplit(editableTile, [nextEditableTile]);
    } else {
      editableTile.getSlider(St11.Side.LEFT)?.onTileSplit(editableTile, [prevEditableTile, nextEditableTile]);
      editableTile.getSlider(St11.Side.RIGHT)?.onTileSplit(editableTile, [prevEditableTile, nextEditableTile]);
      editableTile.getSlider(St11.Side.TOP)?.onTileSplit(editableTile, [prevEditableTile]);
      editableTile.getSlider(St11.Side.BOTTOM)?.onTileSplit(editableTile, [nextEditableTile]);
    }
    this._hoverWidget.handleTileDestroy(editableTile);
    editableTile.destroy();
  }
  deleteTile(editableTile) {
    for (const slider of editableTile.getAllSliders()) {
      if (slider === null)
        continue;
      const success = slider.deleteSlider(editableTile);
      if (success) {
        this._layout.tiles = this._layout.tiles.filter((tile) => tile !== editableTile.tile);
        this._sliders = this._sliders.filter((sl) => sl !== slider);
        this._hoverWidget.handleTileDestroy(editableTile);
        editableTile.destroy();
        slider.destroy();
        return;
      }
    }
  }
  _buildSlider(isHorizontal, coord, groupId) {
    if (!groupId) {
      const groups = this._sliders.map((slider) => slider.groupId).sort();
      groupId = groups.length === 0 ? 1 : groups[groups.length - 1] + 1;
      for (let i = 1; i < groups.length; i++) {
        if (groups[i - 1] + 1 < groups[i]) {
          groupId = groups[i - 1] + 1;
          break;
        }
      }
    }
    return new Slider2(
      this,
      groupId,
      coord,
      coord,
      isHorizontal
    );
  }
  _onDestroy() {
    this.remove_all_children();
    this._sliders.forEach((slider) => slider.destroy());
    this._sliders = [];
    this._minimizedWindows.forEach((win) => win.unminimize());
    super.destroy();
  }
};
LayoutEditor = __decorateClass([
  registerGObjectClass
], LayoutEditor);

// src/indicator/utils.ts
const St12 = imports.gi.St;
const Gio2 = imports.gi.Gio;
const Clutter15 = imports.gi.Clutter;
var createButton = (iconName, text, path) => {
  const btn = new St12.Button({
    styleClass: "message-list-clear-button button",
    canFocus: true,
    xExpand: true,
    child: new St12.BoxLayout({
      vertical: false,
      // horizontal box layout
      clipToAllocation: true,
      xAlign: Clutter15.ActorAlign.CENTER,
      yAlign: Clutter15.ActorAlign.CENTER,
      reactive: true,
      xExpand: true,
      style: "spacing: 8px"
    })
  });
  const icon = new St12.Icon({
    iconSize: 16,
    yAlign: Clutter15.ActorAlign.CENTER
  });
  if (path) {
    icon.gicon = Gio2.icon_new_for_string(`${path}/icons/${iconName}.svg`);
  } else {
    icon.iconName = iconName;
  }
  btn.child.add_child(icon);
  btn.child.add_child(new St12.Label({ marginBottom: 4, marginTop: 4, text, yAlign: Clutter15.ActorAlign.CENTER }));
  return btn;
};

// src/indicator/layoutButton.ts
const Clutter16 = imports.gi.Clutter;
const St13 = imports.gi.St;
var LayoutButtonWidget = class extends LayoutWidget {
  constructor(parent, layout, gapSize, height, width) {
    super({
      parent,
      layout,
      containerRect: buildRectangle({ x: 0, y: 0, width, height }),
      innerGaps: buildMarginOf(gapSize),
      outerGaps: new Clutter16.Margin()
    });
    this.relayout();
  }
  buildTile(parent, rect, gaps, tile) {
    return new SnapAssistTile({ parent, rect, gaps, tile });
  }
};
LayoutButtonWidget = __decorateClass([
  registerGObjectClass
], LayoutButtonWidget);
var LayoutButton = class extends St13.Button {
  constructor(parent, layout, gapSize, height, width) {
    super({
      styleClass: "layout-button button",
      xExpand: false,
      yExpand: false
    });
    parent.add_child(this);
    const [_, scalingFactor] = getScalingFactorOf(this);
    this.child = new St13.Widget();
    new LayoutButtonWidget(this.child, layout, gapSize, height * scalingFactor, width * scalingFactor);
  }
};
LayoutButton = __decorateClass([
  registerGObjectClass
], LayoutButton);

// src/indicator/defaultMenu.ts
const St14 = imports.gi.St;
const Clutter17 = imports.gi.Clutter;
const Main5 = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
var debug13 = logger("DefaultMenu");
var DefaultMenu = class {
  _signals;
  _indicator;
  _layoutsBoxLayout;
  _layoutsButtons;
  _scalingFactor;
  constructor(indicator) {
    this._layoutsButtons = [];
    this._indicator = indicator;
    this._signals = new SignalHandling();
    this._layoutsBoxLayout = new St14.BoxLayout({
      xAlign: Clutter17.ActorAlign.CENTER,
      yAlign: Clutter17.ActorAlign.CENTER,
      xExpand: true,
      yExpand: true,
      vertical: false,
      // horizontal box layout
      styleClass: "layouts-box-layout"
    });
    const layoutsPopupMenu = new PopupMenu.PopupBaseMenuItem({ style_class: "indicator-menu-item" });
    layoutsPopupMenu.add_child(this._layoutsBoxLayout);
    this._indicator.menu.addMenuItem(layoutsPopupMenu);
    this._scalingFactor = getScalingFactorOf(this._layoutsBoxLayout)[1];
    this._drawLayouts();
    this._signals.connect(Settings, Settings.SETTING_LAYOUTS_JSON, () => {
      this._drawLayouts();
    });
    this._signals.connect(Settings, Settings.SETTING_INNER_GAPS, () => {
      this._drawLayouts();
    });
    const buttonsPopupMenu = this._buildEditingButtonsRow();
    this._indicator.menu.addMenuItem(buttonsPopupMenu);
    this._signals.connect(Settings, Settings.SETTING_SELECTED_LAYOUTS, () => {
      const selectedId = Settings.get_selected_layouts()[Main5.layoutManager.primaryIndex];
      const selectedIndex = GlobalState.get().layouts.findIndex((lay) => lay.id === selectedId);
      if (this._layoutsButtons[selectedIndex].checked)
        return;
      this._layoutsButtons.forEach((btn, layInd) => btn.set_checked(layInd === selectedIndex));
    });
    this._signals.connect(Main5.layoutManager, "monitors-changed", () => {
      debug13("monitors-changed");
      this._updateScaling();
    });
  }
  _updateScaling() {
    const newScalingFactor = getScalingFactorOf(this._layoutsBoxLayout)[1];
    if (this._scalingFactor === newScalingFactor)
      return;
    this._scalingFactor = newScalingFactor;
    this._drawLayouts();
  }
  _buildEditingButtonsRow() {
    const buttonsBoxLayout = new St14.BoxLayout({
      xAlign: Clutter17.ActorAlign.CENTER,
      yAlign: Clutter17.ActorAlign.CENTER,
      xExpand: true,
      yExpand: true,
      vertical: false,
      // horizontal box layout
      styleClass: "buttons-box-layout"
    });
    const editLayoutsBtn = createButton("document-edit-symbolic", "Edit Layouts...");
    editLayoutsBtn.connect("clicked", (self) => this._indicator.openLayoutEditor());
    buttonsBoxLayout.add_child(editLayoutsBtn);
    const newLayoutBtn = createButton("add-symbolic", "New Layout...", this._indicator.path);
    newLayoutBtn.connect("clicked", (self) => this._indicator.newLayoutOnClick(true));
    buttonsBoxLayout.add_child(newLayoutBtn);
    const buttonsPopupMenu = new PopupMenu.PopupBaseMenuItem({ style_class: "indicator-menu-item" });
    buttonsPopupMenu.add_child(buttonsBoxLayout);
    return buttonsPopupMenu;
  }
  _drawLayouts() {
    const layouts = GlobalState.get().layouts;
    this._layoutsButtons.forEach((btn) => btn.destroy());
    this._layoutsButtons = [];
    this._layoutsBoxLayout.remove_all_children();
    const hasGaps = Settings.get_inner_gaps(1).top > 0;
    const layoutHeight = 36;
    const layoutWidth = 64;
    this._layoutsButtons = layouts.map((lay, btnInd) => {
      const btn = new LayoutButton(this._layoutsBoxLayout, lay, hasGaps ? 2 : 0, layoutHeight, layoutWidth);
      btn.connect("clicked", (self) => !btn.checked && this._indicator.selectLayoutOnClick(lay));
      return btn;
    });
    const selectedId = Settings.get_selected_layouts()[Main5.layoutManager.primaryIndex];
    const selectedIndex = GlobalState.get().layouts.findIndex((lay) => lay.id === selectedId);
    this._layoutsButtons[selectedIndex]?.set_checked(true);
  }
  destroy() {
    this._signals.disconnect();
    this._layoutsButtons.forEach((btn) => btn.destroy());
    this._layoutsButtons = [];
    this._layoutsBoxLayout.destroy();
    this._indicator.menu.removeAll();
  }
};

// src/indicator/editingMenu.ts
const St15 = imports.gi.St;
const PopupMenu2 = imports.ui.popupMenu;
var EditingMenu = class {
  _indicator;
  constructor(indicator) {
    this._indicator = indicator;
    const boxLayout = new St15.BoxLayout({
      vertical: true,
      styleClass: "buttons-box-layout",
      xExpand: true,
      style: "spacing: 8px"
    });
    const openMenuBtn = createButton("video-display-symbolic", "Menu  ");
    openMenuBtn.connect("clicked", (self) => this._indicator.openMenu(false));
    boxLayout.add_child(openMenuBtn);
    const infoMenuBtn = createButton("info-symbolic", "Info     ", this._indicator.path);
    infoMenuBtn.connect("clicked", (self) => this._indicator.openMenu(true));
    boxLayout.add_child(infoMenuBtn);
    const saveBtn = createButton("done-symbolic", "Save    ", this._indicator.path);
    saveBtn.connect("clicked", (self) => {
      this._indicator.menu.toggle();
      this._indicator.saveLayoutOnClick();
    });
    boxLayout.add_child(saveBtn);
    const cancelBtn = createButton("cancel-symbolic", "Cancel", this._indicator.path);
    cancelBtn.connect("clicked", (self) => {
      this._indicator.menu.toggle();
      this._indicator.cancelLayoutOnClick();
    });
    boxLayout.add_child(cancelBtn);
    const menuItem = new PopupMenu2.PopupBaseMenuItem({ style_class: "indicator-menu-item" });
    menuItem.add_child(boxLayout);
    this._indicator.menu.addMenuItem(menuItem);
  }
  destroy() {
    this._indicator.menu.removeAll();
  }
};

// src/components/editor/editorDialog.ts
const Clutter18 = imports.gi.Clutter;
const St16 = imports.gi.St;
const ModalDialog = imports.ui.modalDialog;
const Main6 = imports.ui.main;
var debug14 = logger("EditorDialog");
var EditorDialog = class extends ModalDialog.ModalDialog {
  _layoutHeight = 72;
  _layoutWidth = 128;
  // 16:9 ratio. -> (16*layoutHeight) / 9 and then rounded to int
  _gapsSize = 3;
  _layoutsBoxLayout;
  constructor(params) {
    super({
      destroyOnClose: true,
      styleClass: "editor-dialog"
    });
    if (params.enableScaling) {
      const monitor = Main6.layoutManager.findMonitorForActor(this);
      const scalingFactor = getScalingFactor(monitor?.index || Main6.layoutManager.primaryIndex);
      enableScalingFactorSupport(this, scalingFactor);
    }
    this.contentLayout.add_child(new St16.Label({
      text: "Select the layout to edit",
      xAlign: Clutter18.ActorAlign.CENTER,
      xExpand: true,
      styleClass: "editor-dialog-title"
    }));
    this._layoutsBoxLayout = new St16.BoxLayout({
      vertical: false,
      // horizontal box layout
      styleClass: "layouts-box-layout",
      xAlign: Clutter18.ActorAlign.CENTER
    });
    this.contentLayout.add_child(this._layoutsBoxLayout);
    if (!params.legend) {
      this._drawLayouts({ layouts: GlobalState.get().layouts, ...params });
    }
    this.addButton({
      label: "Close",
      default: true,
      key: Clutter18.KEY_Escape,
      action: () => params.onClose()
    });
    if (params.legend)
      this._makeLegendDialog({ onClose: params.onClose });
  }
  _makeLegendDialog(params) {
    const suggestion1 = new St16.BoxLayout({ vertical: false });
    suggestion1.add_child(new St16.Label({
      text: "LEFT CLICK",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "button kbd",
      xExpand: false,
      pseudoClass: "active"
    }));
    suggestion1.add_child(new St16.Label({
      text: " to split a tile.",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "",
      xExpand: false
    }));
    const suggestion2 = new St16.BoxLayout({ vertical: false });
    suggestion2.add_child(new St16.Label({
      text: "LEFT CLICK",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "button kbd",
      xExpand: false,
      pseudoClass: "active"
    }));
    suggestion2.add_child(new St16.Label({
      text: " + ",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "",
      xExpand: false
    }));
    suggestion2.add_child(new St16.Label({
      text: "CTRL",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "button kbd",
      xExpand: false,
      pseudoClass: "active"
    }));
    suggestion2.add_child(new St16.Label({
      text: " to split a tile vertically.",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "",
      xExpand: false
    }));
    const suggestion3 = new St16.BoxLayout({ vertical: false });
    suggestion3.add_child(new St16.Label({
      text: "RIGHT CLICK",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "button kbd",
      xExpand: false,
      pseudoClass: "active"
    }));
    suggestion3.add_child(new St16.Label({
      text: " to delete a tile.",
      xAlign: Clutter18.ActorAlign.CENTER,
      yAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "",
      xExpand: false
    }));
    const legend = new St16.BoxLayout({ vertical: true, styleClass: "legend" });
    legend.add_child(suggestion1);
    legend.add_child(suggestion2);
    legend.add_child(suggestion3);
    this.contentLayout.destroy_all_children();
    this.contentLayout.add_child(new St16.Label({
      text: "How to use the editor",
      xAlign: Clutter18.ActorAlign.CENTER,
      xExpand: true,
      styleClass: "editor-dialog-title"
    }));
    this.contentLayout.add_child(legend);
    this.clearButtons();
    this.addButton({
      label: "Start editing",
      default: true,
      key: Clutter18.KEY_Escape,
      action: params.onClose
    });
  }
  _drawLayouts(params) {
    const gaps = Settings.get_inner_gaps(1).top > 0 ? this._gapsSize : 0;
    this._layoutsBoxLayout.destroy_all_children();
    params.layouts.forEach((lay, btnInd) => {
      const box2 = new St16.BoxLayout({
        vertical: true,
        xAlign: Clutter18.ActorAlign.CENTER,
        styleClass: "layout-button-container"
      });
      this._layoutsBoxLayout.add_child(box2);
      const btn = new LayoutButton(box2, lay, gaps, this._layoutHeight, this._layoutWidth);
      if (params.layouts.length > 1) {
        const deleteBtn = new St16.Button({
          xExpand: false,
          xAlign: Clutter18.ActorAlign.CENTER,
          styleClass: "message-list-clear-button icon-button button delete-layout-button"
        });
        deleteBtn.child = new St16.Icon({ iconName: "edit-delete-symbolic", iconSize: 16 });
        deleteBtn.connect("clicked", (self) => {
          params.onDeleteLayout(btnInd, lay);
          this._drawLayouts({ ...params, layouts: GlobalState.get().layouts });
        });
        box2.add_child(deleteBtn);
      }
      btn.connect("clicked", (self) => {
        params.onSelectLayout(btnInd, lay);
        this._makeLegendDialog({ onClose: params.onClose });
      });
      return btn;
    });
    const box = new St16.BoxLayout({
      vertical: true,
      xAlign: Clutter18.ActorAlign.CENTER,
      styleClass: "layout-button-container"
    });
    this._layoutsBoxLayout.add_child(box);
    const newLayoutBtn = new LayoutButton(box, new Layout([new Tile2({ x: 0, y: 0, width: 1, height: 1, groups: [] })], "New Layout"), gaps, this._layoutHeight, this._layoutWidth);
    const icon = new St16.Icon({ iconName: "list-add-symbolic", iconSize: 32 });
    icon.set_size(newLayoutBtn.child.width, newLayoutBtn.child.height);
    newLayoutBtn.child.add_child(icon);
    newLayoutBtn.connect("clicked", (self) => {
      params.onNewLayout();
      this._makeLegendDialog({ onClose: params.onClose });
    });
  }
};
EditorDialog = __decorateClass([
  registerGObjectClass
], EditorDialog);

// src/indicator/indicator.ts
const Gio3 = imports.gi.Gio;
const St17 = imports.gi.St;
const Shell4 = imports.gi.Shell;
const Main7 = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
var debug15 = logger("indicator");
var IndicatorState = /* @__PURE__ */ ((IndicatorState2) => {
  IndicatorState2[IndicatorState2["DEFAULT"] = 1] = "DEFAULT";
  IndicatorState2[IndicatorState2["CREATE_NEW"] = 2] = "CREATE_NEW";
  IndicatorState2[IndicatorState2["EDITING_LAYOUT"] = 3] = "EDITING_LAYOUT";
  return IndicatorState2;
})(IndicatorState || {});
var Indicator3 = class extends PanelMenu.Button {
  _layoutEditor;
  _editorDialog;
  //@ts-ignore todo
  _currentMenu;
  _state;
  _enableScaling;
  _path;
  constructor(path, uuid) {
    super(0.5, "Tiling Shell Indicator", false);
    Main7.panel.addToStatusArea(uuid, this, 1, "right");
    Settings.bind(Settings.SETTING_SHOW_INDICATOR, this, "visible");
    const icon = new St17.Icon({
      gicon: Gio3.icon_new_for_string(`${path}/icons/indicator-symbolic.svg`),
      styleClass: "system-status-icon indicator-icon"
    });
    this.add_child(icon);
    this._layoutEditor = null;
    this._editorDialog = null;
    this._state = 1 /* DEFAULT */;
    this._enableScaling = false;
    this._path = path;
    this.connect("destroy", this._onDestroy.bind(this));
  }
  get path() {
    return this._path;
  }
  set enableScaling(value) {
    if (this._enableScaling === value)
      return;
    this._enableScaling = value;
    if (value) {
      const monitor = Main7.layoutManager.findMonitorForActor(this);
      const scalingFactor = getScalingFactor(monitor?.index || Main7.layoutManager.primaryIndex);
      enableScalingFactorSupport(this.menu.box, scalingFactor);
    }
    if (this._currentMenu && this._state === 1 /* DEFAULT */) {
      this._currentMenu.destroy();
      this._currentMenu = new DefaultMenu(this);
    }
  }
  enable() {
    this.menu.removeAll();
    this._currentMenu = new DefaultMenu(this);
  }
  selectLayoutOnClick(layoutToSelect) {
    Settings.save_selected_layouts_json(getMonitors().map((monitor) => layoutToSelect.id));
    this.menu.toggle();
  }
  newLayoutOnClick(showLegendOnly) {
    this.menu.close(true);
    const newLayout = new Layout([
      new Tile2({ x: 0, y: 0, width: 0.3, height: 1, groups: [1] }),
      new Tile2({ x: 0.3, y: 0, width: 0.7, height: 1, groups: [1] })
    ], `${Shell4.Global.get().get_current_time()}`);
    if (this._layoutEditor)
      this._layoutEditor.layout = newLayout;
    else
      this._layoutEditor = new LayoutEditor(newLayout, Main7.layoutManager.monitors[Main7.layoutManager.primaryIndex], this._enableScaling);
    this._setState(2 /* CREATE_NEW */);
    if (showLegendOnly)
      this.openMenu(true);
  }
  openMenu(showLegend) {
    if (this._editorDialog)
      return;
    this._editorDialog = new EditorDialog({
      enableScaling: this._enableScaling,
      onNewLayout: () => {
        this.newLayoutOnClick(false);
      },
      onDeleteLayout: (ind, lay) => {
        GlobalState.get().deleteLayout(lay);
        if (this._layoutEditor && this._layoutEditor.layout.id === lay.id) {
          this.cancelLayoutOnClick();
        }
      },
      onSelectLayout: (ind, lay) => {
        const layCopy = new Layout(lay.tiles.map((t) => new Tile2({ x: t.x, y: t.y, width: t.width, height: t.height, groups: [...t.groups] })), lay.id);
        if (this._layoutEditor)
          this._layoutEditor.layout = layCopy;
        else
          this._layoutEditor = new LayoutEditor(layCopy, Main7.layoutManager.monitors[Main7.layoutManager.primaryIndex], this._enableScaling);
        this._setState(3 /* EDITING_LAYOUT */);
      },
      onClose: () => {
        this._editorDialog?.destroy();
        this._editorDialog = null;
      },
      legend: showLegend
    });
    this._editorDialog.open();
  }
  openLayoutEditor() {
    this.openMenu(false);
  }
  saveLayoutOnClick() {
    if (this._layoutEditor === null || this._state === 1 /* DEFAULT */)
      return;
    const newLayout = this._layoutEditor.layout;
    if (this._state === 2 /* CREATE_NEW */) {
      GlobalState.get().addLayout(newLayout);
    } else {
      GlobalState.get().editLayout(newLayout);
    }
    this.menu.toggle();
    this._layoutEditor.destroy();
    this._layoutEditor = null;
    this._setState(1 /* DEFAULT */);
  }
  cancelLayoutOnClick() {
    if (this._layoutEditor === null || this._state === 1 /* DEFAULT */)
      return;
    this.menu.toggle();
    this._layoutEditor.destroy();
    this._layoutEditor = null;
    this._setState(1 /* DEFAULT */);
  }
  _setState(newState) {
    if (this._state === newState)
      return;
    this._state = newState;
    this._currentMenu.destroy();
    switch (newState) {
      case 1 /* DEFAULT */:
        this._currentMenu = new DefaultMenu(this);
        break;
      case 2 /* CREATE_NEW */:
      case 3 /* EDITING_LAYOUT */:
        this._currentMenu = new EditingMenu(this);
        break;
    }
  }
  _onDestroy() {
    this._editorDialog?.destroy();
    this._layoutEditor?.destroy();
    this._layoutEditor = null;
    this._currentMenu.destroy();
  }
};
Indicator3 = __decorateClass([
  registerGObjectClass
], Indicator3);

// src/extension.ts
const Main8 = imports.ui.main;
const Gio4 = imports.gi.Gio;

var SIGNAL_WORKAREAS_CHANGED = "workareas-changed";
var debug16 = logger("extension");
var TilingShellExtension = class extends Extension {
  _indicator;
  _tilingManagers;
  _fractionalScalingEnabled;
  _mutterSettings;
  _dbus;
  _signals;
  constructor(metadata) {
    super(metadata);
    this._signals = null;
    this._fractionalScalingEnabled = false;
    this._mutterSettings = null;
    this._tilingManagers = [];
    this._indicator = null;
    this._dbus = null;
  }
  createIndicator() {
    this._indicator = new Indicator3(this.path, this.uuid);
    this._indicator.enableScaling = !this._fractionalScalingEnabled;
    this._indicator.enable();
  }
  _validateSettings() {
    if (this.metadata.version) {
      Settings.set_last_version_installed(Number(this.metadata.version));
    }
    const selectedLayouts = Settings.get_selected_layouts();
    const monitors = getMonitors();
    const layouts = GlobalState.get().layouts;
    if (selectedLayouts.length === 0)
      selectedLayouts.push(layouts[0].id);
    while (monitors.length < selectedLayouts.length) {
      selectedLayouts.pop();
    }
    while (monitors.length > selectedLayouts.length) {
      selectedLayouts.push(selectedLayouts[0]);
    }
    for (let i = 0; i < selectedLayouts.length; i++) {
      if (layouts.findIndex((lay) => lay.id === selectedLayouts[i]) === -1) {
        selectedLayouts[i] = selectedLayouts[0];
      }
    }
    Settings.save_selected_layouts_json(selectedLayouts);
  }
  enable() {
    if (this._signals)
      this._signals.disconnect();
    this._signals = new SignalHandling();
    Settings.initialize(this.getSettings());
    this._validateSettings();
    this._mutterSettings = new Gio4.Settings({ schema: "org.gnome.mutter" });
    this._fractionalScalingEnabled = this._isFractionalScalingEnabled();
    if (Main8.layoutManager._startingUp) {
      this._signals.connect(Main8.layoutManager, "startup-complete", () => {
        this._createTilingManagers();
        this._setupSignals();
      });
    } else {
      this._createTilingManagers();
      this._setupSignals();
    }
    this.createIndicator();
    this._dbus = Gio4.DBusExportedObject.wrapJSObject(
      `<node>
        <interface name="org.gnome.Shell.Extensions.TilingShell">
          <method name="openLayoutEditor" />
        </interface>
      </node>`,
      this
    );
    this._dbus.export(Gio4.DBus.session, "/org/gnome/shell/extensions/TilingShell");
    debug16("extension is enabled");
  }
  openLayoutEditor() {
    this._indicator?.openLayoutEditor();
  }
  _createTilingManagers() {
    debug16("building a tiling manager for each monitor");
    this._tilingManagers.forEach((tm) => tm.destroy());
    this._tilingManagers = getMonitors().map((monitor) => new TilingManager(monitor, !this._fractionalScalingEnabled));
    this._tilingManagers.forEach((tm) => tm.enable());
  }
  _setupSignals() {
    if (!this._signals)
      return;
    this._signals.connect(global.display, SIGNAL_WORKAREAS_CHANGED, () => {
      const allMonitors = getMonitors();
      if (this._tilingManagers.length !== allMonitors.length) {
        const oldIndexes = Settings.get_selected_layouts();
        const indexes = allMonitors.map((monitor) => {
          if (monitor.index >= oldIndexes.length)
            return GlobalState.get().layouts[0].id;
          return oldIndexes[monitor.index];
        });
        Settings.save_selected_layouts_json(indexes);
      }
      this._createTilingManagers();
    });
    if (this._mutterSettings) {
      this._signals.connect(this._mutterSettings, "changed::experimental-features", () => {
        if (!this._mutterSettings)
          return;
        const fractionalScalingEnabled = this._isFractionalScalingEnabled();
        if (this._fractionalScalingEnabled === fractionalScalingEnabled)
          return;
        this._fractionalScalingEnabled = fractionalScalingEnabled;
        this._createTilingManagers();
        if (this._indicator)
          this._indicator.enableScaling = !this._fractionalScalingEnabled;
      });
    }
  }
  _isFractionalScalingEnabled() {
    if (!this._mutterSettings)
      return false;
    return this._mutterSettings.get_strv("experimental-features").find((feat) => feat === "scale-monitor-framebuffer" || feat === "x11-randr-fractional-scaling") != void 0;
  }
  disable() {
    this._indicator?.destroy();
    this._indicator = null;
    this._tilingManagers.forEach((tm) => tm.destroy());
    this._tilingManagers = [];
    if (this._signals)
      this._signals.disconnect();
    this._signals = null;
    GlobalState.destroy();
    this._mutterSettings = null;
    this._fractionalScalingEnabled = false;
    this._dbus?.flush();
    this._dbus?.unexport();
    this._dbus = null;
    Settings.destroy();
    debug16("extension is disabled");
  }
};


function init(meta) {
    return new TilingShellExtension(meta);
}
